/******************************************************************************************************************
Revision: 1    
Revision date: May 31 08
Author: Red Squirrel
Tested in devc++(win32) and g++(Linux)



These are common functions and classes used in various apps - should work in linux and windows
*********************************************************************************************************************
How to use this header:

	#include <rslib/_allrslib.h>
	using namespace rslib;


	features that need to be enabled:
	
	rslib_pthread 	//enable pthread
	rslib_network	//enable networking
	rslib_mysql 	//enable mysql (requires mysql++ working and enables networking)	
	
	ex:

	#define rslib_network
	#include <rslib/_allrslib.h>
	using namespace rslib;
	
	Also need to add winsock library in windows (libwsock32.a)

DO NOT include any of the below headers or statements as this header takes care of it
***********************************************************************************************************************/



#include <iostream>
#include <fstream>
#include <vector>
#include <deque>
#include <iomanip>
#include <sstream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h> 
#include <cstring> 
#include <sys/time.h>
#include <unistd.h>



using namespace std;

#ifdef WIN32
    #include <windows.h>
    #include <conio.h>
	#include <unistd.h>
	string DIRSPACER="\\";
#endif
#ifdef linux
    #include <termios.h>
	string DIRSPACER="/";
#endif





//socket stuff:

#ifdef rslib_network

    #include <errno.h>
	#include <fcntl.h>
	
	#ifdef WIN32
        #include <winsock2.h>
		#include <ws2tcpip.h>		
		#define MSG_NOSIGNAL 0
		#define MSG_DONTWAIT 0
    #endif
    #ifdef linux
        #include <sys/poll.h>    
        #include <unistd.h>
        #include <sys/socket.h>
        #include <sys/types.h>
		#include <sys/resource.h>
        #include <arpa/inet.h>
        #include <netinet/in.h>
        #include <netdb.h>    	
        #define INVALID_SOCKET -1
        #define SOCKET_ERROR -1	
    #endif
#endif

#ifdef rslib_pthread
	#include <pthread.h>
#endif

//mysql stuff:

#ifdef rslib_mysql
#include <mysql++/mysql++.h> 
#endif

typedef unsigned long int uli;













/*
NOTE: Cleanup is in progress, this file should only contain references to the other header files!

*/



	/*+++++++++++++++++++++++
	 Prompting/conversion and related functions
	 ++++++++++++++++++++++++*/





	string Uint2StrHex(unsigned long int sourceint)
	{
	stringstream ss;

	ss <<"0x"<<hex<<sourceint;

	string val;
	ss >> val;
	return val;	
	}
	
	string Int2StrHex(long int sourceint)
	{
	 stringstream ss;

	ss <<"0x"<<hex<<sourceint;

	string val;
	ss >> val;
	return val;	
	}	


	//turns long int to string
	string Int2Str(long int sourceint)
	{
	 stringstream ss;

	ss << sourceint;

	string val;
	ss >> val;
	return val;
	}	
	
	//turns unsigned long int to string
	string Uint2Str(unsigned long int sourceint)
	{
	 stringstream ss;

	ss << sourceint;

	string val;
	ss >> val;
	return val;
	}	


	//turns string to int
	int Str2Int(string sourcestr)
	{
	return atoi(sourcestr.c_str());
	}

	//turns str to bool
	bool Str2Bool(string source)
	{
	if(source.size()<1)return false;
	if(tolower(source[0])=='y' || tolower(source[0])=='t' || tolower(source[0])=='1')return true;
	return false;
	}









	//PROTOTYPES:	

	#ifdef rslib_pthread
	#include "pthread.h"
	#endif
	

	#include "containers.h"
	#include "strmanip.h"
	#include "args.h"
	#include "userio.h"
	#include "random.h"
	#include "misc.h"
	#include "bitstream.h"
	#include "bytestream.h"	
	#include "configfile.h"
	#include "datetime.h"
	#include "mathstuff.h"
	#include "csvreader.h"
	#include "smartpointers.h"

	#ifdef rslib_network
	#include "network.h"
	#endif	

	#ifdef rslib_mysql
	#include "mysql.h"
	#endif


	//IMPLIMENTATORS:

	#ifdef rslib_pthread
	#include "pthread.cpp"
	#endif
	
	#include "containers.cpp"
	#include "strmanip.cpp"
	#include "args.cpp"
	#include "userio.cpp"
	#include "random.cpp"
	#include "misc.cpp"
	#include "bitstream.cpp"
	#include "bytestream.cpp"	
	#include "configfile.cpp"	
	#include "datetime.cpp"	
	#include "mathstuff.cpp"
	#include "csvreader.cpp"
	#include "smartpointers.cpp"
	
	#ifdef rslib_network
	#include "network.cpp"
	#endif

	#ifdef rslib_mysql
	#include "mysql.cpp"
	#endif	









