/*
======================================================
rslib component:  argument handler implimentors
======================================================
*/



namespace rslib
{

    ClArgs::ClArgs()
	{
	}
	
	
	ClArgs::ClArgs(int argc, char* argv[],string pextracompileargs="")
    {
        extracompileargs=pextracompileargs;
        string tmp;
        ArgList tmp2;
        int tmpfind;
        bool inquotes=false;
        string inquotesvar;
        string inquotesvalue;
        
        fullprogrampath=argv[0];		
		programpath=DefineProgramPath();
		programname=DefineProgramName();
		
		string argfile = programname + "_args.txt";
		
		fstream fin(argfile.c_str(),ios::in);
		
		if(fin)
		{
			if(extracompileargs.size()>0)extracompileargs.append(1,' ');
			
			char tmpargchar;
			
			while(!fin.eof())
			{
				tmpargchar=fin.get();
				
				if(tmpargchar>126 || tmpargchar<32)continue;
				
				extracompileargs.append(1,tmpargchar);
			}
		}
		else
		{
			fstream fout(argfile.c_str(),ios::out);
			fout.close();
		}
		
		fin.close();
		
        
        if(fullprogrampath[0]=='.' && fullprogrampath[1]=='/')fullprogrampath=fullprogrampath.substr(2,fullprogrampath.size()-2);
        
        for(int i=1;i<argc;i++)
        {
            tmp=argv[i];
            tmp2.argument="";
            tmp2.value="";
                        
            if(tmp[0]!='-' && !inquotes)continue; //ignore stuff that does not start with -
            
            if(inquotes)
            {
                inquotesvalue+=tmp;
                if(inquotesvalue[inquotesvalue.size()-1]=='"')
                {
                    inquotes=false;
                    tmp2.argument=inquotesvar;
                    tmp2.argument=inquotesvalue;
                    arglist.push_back(tmp2);
                }   
            }
            else
            {
                tmp2.argument=tmp.substr(1,tmp.size()-1);
                                
                tmpfind=tmp2.argument.find("=");
                if(tmpfind!=string::npos && tmpfind<tmp2.argument.size())
                {
                    tmp2.value=tmp2.argument.substr(tmpfind+1,tmp2.argument.size()-tmpfind);
                    tmp2.argument=tmp2.argument.substr(0,tmpfind);
                }
                        
                if(tmp2.value[0]=='"')
                {
                    inquotes=true; 
                    inquotesvar=tmp2.argument;
                    inquotesvalue=tmp2.value.substr(1,tmp2.value.size()-1);
                }
                else arglist.push_back(tmp2);
            }
        }
    }




    bool ClArgs::HasArg(string arg)
    {
        for(int i=0;i<arglist.size();i++)if(arg==arglist[i].argument)return true;
        return false;
    }   
    
    
    
    string ClArgs::GetValue(string arg)
    {
         for(int i=0;i<arglist.size();i++)if(arg==arglist[i].argument)return arglist[i].value;
         return "";
    }
	
	int ClArgs::GetValueInt(string arg)
	{
		return BitStream::Str2Int(GetValue(arg));
	}
    
	
	void ClArgs::SetValue(string arg,string value)
	{
		ArgList tmp;
		tmp.argument=arg;
		tmp.value=value;
		arglist.push_back(tmp);
	}

    
    string ClArgs::ShowArgs(string spacer,int ignoreflags)
    {
        string ret="";     
        for(int i=0;i<arglist.size();i++)
        {
            if((ignoreflags==1 || ignoreflags==3) && arglist[i].argument=="compile")continue;
            else if((ignoreflags==2 || ignoreflags==3) && arglist[i].argument=="debug")continue;
            
            ret+=spacer + "-" + arglist[i].argument; 
            
            if(arglist[i].value!="")
            {
                ret+="=" + arglist[i].value;
            }            
        }
        return ret;
    }
    




    bool ClArgs::HandleDebug()
    {
        if(!HasArg("debug"))return false;
        
        #ifdef WIN32
        cout<<endl<<"-debug not available in win32"<<endl;
        return false;
        #endif
        
        fstream fin(".gdbinit",ios::in);
        
        if(!fin)
        {
            fstream fout(".gdbinit",ios::out);
			system("chmod 700 .gdbinit >/dev/null");
			system("chown root .gdbinit >/dev/null");
            fout<<"run\nbt\nquit\n";
            fout.close();
        }
        fin.close();
        
        string exec="gdb -silent -args " + fullprogrampath + ShowArgs(" ",2);
        
        system(exec.c_str());
        
        return true;
    }
    
    
    bool ClArgs::HandleCompile()
    {
        if(!HasArg("compile"))return false;
        
        #ifdef WIN32
        cout<<endl<<"-compile not available in win32"<<endl;
        return false;
        #endif   
		
        
        string exec="g++ -o " + fullprogrampath + " " + fullprogrampath + ".cpp";
        if(HasArg("debug"))exec+=" -g ";
        else exec+=" " + GetValue("compile");
        exec+=" " + extracompileargs;		

        system("clear");
        cout<<endl<<endl<<flush<<"Recompiling ["<<exec<<"] ...";
        cout<<endl<<"*******************************************************************************"<<endl<<endl;
                        
        int comperr=system(exec.c_str());
        
        cout<<endl<<"*******************************************************************************"<<endl;
        
        if(comperr!=0)
        {
            cout<<"Completed.  ERROR Compiling."<<endl<<endl<<endl; 
			Wait(5000000);
            return false;           
        }
        else
        {
            cout<<"Completed.  Relaunching..."<<endl<<endl<<endl;
            system(("./" + fullprogrampath + ShowArgs(" ",1)).c_str());
			
			fstream fout(("rebuild_" + programname).c_str(),ios::out);
			fout<<exec<<endl;
			fout.close();
			system(("chmod 700 rebuild_" + programname).c_str());
			
			Wait(1000000);
            return true;
        }

        return false;
    }
    
    
    bool ClArgs::HandleSpecialArgs()
    {
        if(HandleCompile())return true;
        return HandleDebug();
    }
	
	
	string ClArgs::DefineProgramPath()
	{
		string ret="";		
		int pos=0;
		
		pos=fullprogrampath.find_last_of("/");
		if(pos==string::npos)return "";			
		
		return fullprogrampath.substr(0,pos+1);
	}
		
	
	string ClArgs::DefineProgramName()
	{
		string ret=fullprogrampath;
		
		int fpos=0;
		while(1)
		{		
			fpos=ret.find("/",0);
			if(fpos==string::npos)break;
			ret=ret.substr(fpos+1,ret.size()-(fpos+1));
		}
		
		return ret;
	}	
	
	
	
	string ClArgs::GetProgramPath(bool showexec=false)
	{		
		return programpath + (showexec ? programname : "");
	}
	
	string ClArgs::GetProgramName()
	{
		return programname;
	}

}

