/*
======================================================
rslib component:  argument handler prototypes
======================================================
*/


#ifndef args_h
#define args_h


/**
This header used for better handling command line arguments and also handling stuff like debug with gdb


To initiate ensure your main() looks like this:

||||||||||||||||||||||||||||||||||||||||||

int main(int argc, char* argv[])
{
    ClArgs args(argc,argv,"");
    if(args.HandleSpecialArgs())return 0;
	//----------  Program start:  -------
	
	
	return 0;
}

||||||||||||||||||||||||||||||||||||||||||

NOTE: You can also create a file called programname_args.txt with the arguments in it


*/

namespace rslib
{
    class ClArgs
    {
        struct ArgList
        {
            string argument;
            string value;
        };        
        
        private: 
            vector<ArgList> arglist;
            string fullprogrampath;  //full path with name
			string programpath;		//program path (GetProgramPath() returns this)
			string programname;		//program name only (GetProgramName() returns this)
            string extracompileargs;
            bool HandleDebug();   //if returns false keep executing, if true restart app
            bool HandleCompile(); //if returns false keep executing, if true restart app     
			string DefineProgramPath(); //defines var for program path
			string DefineProgramName(); //defines var for program name			
	
        
        public:
			ClArgs();
            ClArgs(int argc, char** argv,string);
            bool HasArg(string arg);			//returns true of arg exists
            string GetValue(string arg);		//gets value of arg
            int GetValueInt(string arg);		//gets int value of arg
			void SetValue(string arg,string val);	//sets value of arg (useful for when args need to be set/modified and used within CLarg object)
            string ShowArgs(string spacer="\n  ",int ignoreflags=0);			//show all arguments - //ignoreflags: 1= compile, 2=debug, 3=both
            bool HandleSpecialArgs(); 				//calls handledebug and handlecompile in proper order
			string GetProgramPath(bool showexec); 	//gets path to program (true for executable name too)
			string GetProgramName(); 				//returns executable name	

    };
}



#endif


