/*
======================================================
rslib component:  bitstream implimentors
======================================================
*/


namespace rslib
{

	BitStream::Node::Node()
	{
		prev=NULL;
		next=NULL;
	}
	
	BitStream::Node::Node(Node * prevN,Node * nextN)
	{
		prev=prevN;
		next=nextN;
		
		if(prevN!=NULL)prevN->next=this;
		if(nextN!=NULL)nextN->prev=this;		
	}


	BitStream::BitStream()
    {
		m_bitsize=0;		
		m_start=NULL;
		m_end=NULL;
    }
	
	BitStream::BitStream(const BitStream &copy)
	{
		m_bitsize=0;		
		m_start=NULL;
		m_end=NULL;
		
		Node * reading = copy.m_start;
				
		while(reading!=NULL)
		{
			Node * toadd = AddNode();
			toadd->data=reading->data;
			
			reading = GetNextNode(reading);
			if(reading==NULL)break;
		}
	}
	
	BitStream::BitStream(const string &copy)
	{
		m_bitsize=0;		
		m_start=NULL;
		m_end=NULL;
		
		WriteString(copy);
	}	
	
	BitStream::~BitStream()
	{
		Clear();
	}
	

	
	BitStream::Node * BitStream::AddNode(bool start)
	{
		Node * newnode=NULL;		
		
		if(m_start==NULL) 		//this is the first node ever added
		{			
			m_bitsize++;
			newnode = new Node();
			m_start=newnode;
			m_end=newnode;
		}		
		else if(start)  			//we want to add the node at the start
		{			
			m_bitsize++;
			newnode = new Node(NULL,m_start);
			m_start=newnode;			
		}		
		else					//we want to add at the end
		{			
			m_bitsize++;
			newnode = new Node(m_end,NULL);
			m_end=newnode;		
		}
		return newnode;
	}
	
	BitStream::Node * BitStream::InsertNodeAfter(Node * ref)
	{
		Node * newnode = NULL;
		
		if(m_start==NULL)			//if start is null, then this is the first node ever
		{
			newnode = AddNode();
		}
		else if(ref==NULL)			//if ref is null, assume we want to add at the start
		{
			newnode = AddNode(true);
		}
		else if(ref->next==NULL)  	//if "next" is null then it means ref is the last node, new node is now last
		{
			m_bitsize++;
			newnode = new Node(ref,NULL);
			m_end=newnode;	
		}
		else							//standard insert after ref node
		{
			m_bitsize++;
			newnode = new Node(ref,ref->next);
		}
		return newnode;
	}
	
	BitStream::Node * BitStream::InsertNodeBefore(Node * ref)
	{
		Node * newnode = NULL;
		
		if(m_start==NULL)			//if start is null, then this is the first node ever
		{
			newnode = AddNode();
		}
		else if(ref==NULL)			//if ref is null, assume we want to add at the start
		{
			newnode = AddNode(true);
		}
		else if(ref->prev==NULL)  	//if "prev" is null then it means ref is the first node, new node is now first
		{
			m_bitsize++;
			newnode = new Node(NULL,ref);
			m_start=newnode;	
		}
		else							//standard insert after ref node
		{
			m_bitsize++;
			newnode = new Node(ref->prev,ref);
		}
		return newnode;
	}	
	
	void BitStream::DelNode(BitStream::Node * ref)
	{
		if(m_bitsize<=0 || ref==NULL)return;
		
		
		if(ref->prev==NULL && ref->next==NULL)  //this node is both the start and the end (aka, only one node in the system)
		{
			m_start=NULL;
			m_end=NULL;
		}
		else if(ref->prev==NULL)	//this node is the start, and there are other nodes after
		{
			m_start=ref->next;
			ref->next->prev=NULL;		
		}
		else if(ref->next==NULL) //this is the last node
		{
			m_end=ref->prev;
			ref->prev->next=NULL;
		}
		else //this node is in the middle
		{
			ref->prev->next = ref->next;
			ref->next->prev = ref->prev;
		}
		m_bitsize--;
		delete ref;
	}
    
	BitStream::Node * BitStream::GetNextNode(BitStream::Node * ref)
	{
		if(ref==NULL)return m_start;			//if argument is null, then we are starting from the beginning		
		else if(ref==m_end)return NULL; 		//if we're at the end, just return null
		
		return ref->next;
	}
	
	BitStream::Node * BitStream::GetPrevNode(BitStream::Node * ref)
	{
		if(ref==NULL)return m_end;			//if argument is null, then we are starting from the end	
		else if(ref==m_start)return NULL; 		//if we're at the start, just return null
		
		return ref->prev;
	}	
	
	BitStream::Node * BitStream::GetNode(bsint pos)
	{
		if(pos>=m_bitsize)return NULL; //if position is past end we return null
		
		Node * ret=NULL;
		
		if(pos==0)return m_start; //if pos is 0 then we want start
		else if(pos==m_bitsize-1)return m_end; //if pos is size-1 then we want end
		
		//we want a node in middle of list:
		
		//determine what is faster, seeking from start to node or from end to node
		
		if(pos>(m_bitsize/2))	//seek from end to node
		{
			ret=m_end;
			for(bsint i=m_bitsize-1;i!=pos;i--)
			{				
				ret = GetPrevNode(ret);
			}
		}
		else	//seek from start to node
		{		
			ret=m_start;
			for(bsint i=0;i<pos;i++)
			{
				ret = GetNextNode(ret);
			}
		}
		
		return ret;
	}
	
	
	
    void BitStream::WriteBit_internal(bool tw,Node * ref,bool start)
    {
        Node * added=NULL;
		if(ref!=NULL)  //use reference node
		{
			added = InsertNodeBefore(ref);		
		}
		else //add to start or end of list
		{
			added = AddNode(start);
		}
		
		added->data=tw;
    }
    
    
    
    //writes a int based on specified lenght (bits)
    void BitStream::WriteVarInt_internal(bsint tw,int bits,Node * ref,bool start)
    {
    	if(bits>MaxBits())bits=MaxBits();
    	if(bits<1)bits=1;
		
		
    
    	//orer is used to determine the number we need to "OR" with for each bit, to construct the individual bits of the int
		BitStream::bsint orer=1;	
    	for(int i=1;i<bits;i++)
    	{
    	   orer*=2;
    	}
    	
    	//make sure tw is not past its capacity
    	tw = (tw>((orer*2)-1)) ? ((orer*2)-1) : tw;		
		
		
		//loop through each bit using OR so we can generate each bit
		Node * added=NULL;
    	
    	for(int i=0;i<bits;i++)
    	{
        	if(added==NULL) //we do not have a reference node, so we create it here
			{
				if(ref==NULL) added = AddNode(start);
				else added = InsertNodeBefore(ref);
			}
			else added = InsertNodeAfter(added);
			
			
			added->data = (((orer & tw)==orer) ? true : false); 
        	orer/=2;
    	}
    }
	

	void BitStream::WriteString_internal(string tw,Node * ref,bool start)
	{
		Node * addnode=NULL;		
		
		for(int i=0;i<tw.size();i++)
		{			
			//first node of first byte is a bit different...
			
			if(addnode==NULL) //if this is the first node of the bs we are adding, do some priliminary checking/setting
			{
				if(ref==NULL)addnode = AddNode(start); //add node at start or end		
				else addnode = InsertNodeBefore(ref); //add before ref
				
			}
			else addnode = InsertNodeAfter(addnode);
					
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 128;
			
			//do rest of nodes...
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 64;
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 32;						
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 16;						
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 8;						
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 4;						
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 2;						
			
			addnode = InsertNodeAfter(addnode);			
			addnode->data = ((unsigned int)(unsigned char)tw[i]) & 1;
			
			//one byte completed, continue on...
		}
		
	}
	
	
	
	void BitStream::WriteBitStream_internal(BitStream tw,Node * ref,bool start)
	{
		Node * twnode=NULL;
		Node * addnode=NULL;
		
		while(1)
		{
			twnode = tw.GetNextNode(twnode);
			if(twnode==NULL)break;
			
			
			if(addnode==NULL) //if this is the first node of the bs we are adding, do some priliminary checking/setting
			{
				if(ref==NULL)
				{
					addnode = AddNode(start); 	//add node at start or end	
				}
				else
				{
					addnode = InsertNodeBefore(ref); //add first node after reference
				}
				
				addnode->data=twnode->data;
				continue;
			}
			
			
			addnode = InsertNodeAfter(addnode);			//add node using last added node 
			
			addnode->data=twnode->data;
		}	
	}
	
	
    BitStream::bsint BitStream::ReadVarInt_internal(int bits,Node * ref,bool start,bool pop)
    {    	
    	
		BitStream::bsint tmp=0;    	
    	BitStream::bsint orer=1;	
    	for(int i=1;i<bits;i++)
    	{
    	  orer*=2;
    	}
		
		if(ref==NULL)
		{
			if(start || (bits>=m_bitsize))ref=m_start;
			else ref = GetNode(m_bitsize-bits);
			
			//there's still a chance for ref to be null here so we check again and return 0 if it is.
			if(ref==NULL)return 0;
		}
		
		Node * tmpnode=ref;
    	
		for(int i=0;i<bits;i++)
    	{    					
			if(tmpnode==NULL)break;
			
			if(tmpnode->data)tmp |=orer;
    		orer/=2;
			tmpnode = GetNextNode(tmpnode);
    	}
		
				
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<(bits);i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}		
    
    	return tmp;
    }	
	
	
	string BitStream::ReadString_internal(bsint chars,Node * ref,bool start,bool pop)
	{
		string ret;    
		Node * tmpnode=NULL;
    	bool byte[8] = {0,0,0,0,0,0,0,0}; //used to store 8 bit chunks    
    	unsigned int ibyte; //used to construct a byte as a number    
		
		
		if(ref==NULL)
		{
			if(start || ((chars*8)>=m_bitsize))ref=m_start;
			else ref = GetNode(m_bitsize-(chars*8));
			
			//there's still a chance for ref to be null here so we check again and return nothing if it is.
			if(ref==NULL)return "";
		}
		
		Node * tmp = ref; 
    
    	for(int i=0;i<chars;i++)
    	{
            ibyte=0;
			
			//get one byte:
			//-------------
			
			for(int ii=0;ii<8;ii++)
			{				 
				byte[0+ii]=tmp->data;
				tmp = GetNextNode(tmp);
			}  
    		
    		//do stuff with bits and stuff:
			
			if(byte[0])ibyte |=128;	
    		if(byte[1])ibyte |=64;	
    		if(byte[2])ibyte |=32;	
    		if(byte[3])ibyte |=16;
    		if(byte[4])ibyte |=8;	
    		if(byte[5])ibyte |=4;	
    		if(byte[6])ibyte |=2;	
    		if(byte[7])ibyte |=1;	
    
			ret.append(1,(unsigned char)ibyte);
    	}
		
		
		
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<(chars*8);i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}	
		
    
    	return ret;
	}
	
	
	BitStream BitStream::ReadBitStream_internal(bsint bits,Node * ref,bool start,bool pop)
	{
		BitStream ret;    
		Node * tmpnode=NULL;
		
		if(ref==NULL)
		{
			if(start || (bits>=m_bitsize))ref=m_start;
			else ref = GetNode(m_bitsize-bits);
			
			//there's still a chance for ref to be null here so we check again and return nothing if it is.
			if(ref==NULL)return ret;
		}
		
		Node * tmp = ref; 
    
    	for(int i=0;i<bits;i++)
    	{            			 
			ret.WriteBit_internal(tmp->data,NULL);
			tmp = GetNextNode(tmp);			
    	}
		
		
		
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<bits;i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}	
		
    
    	return ret;
	}
	
	
	

	bool BitStream::Lock(bool block=true)
	{
		#ifdef rslib_pthread
		if(block)mutex.Lock();
		else return mutex.TryLock();
		#endif

		return true;
	}

	
	void BitStream::Unlock()
	{
		#ifdef rslib_pthread
		mutex.UnLock();
		#endif
	}
	
	

    string BitStream::ToStr()
    {
    	string temp;    
    
    	bool byte[8] = {0,0,0,0,0,0,0,0}; //used to store 8 bit chunks    
    	unsigned int ibyte; //used to construct a byte as a number    
		
		Node * tmp = NULL;
    
        //Note: this assumes a bitstream with multiples of 8. Any extra bits are dropped.    
    
    	for(int i=0;(i+8)<=m_bitsize;i+=8)
    	{
            
            ibyte=0;
			
			//get one byte:
			//-------------
			
			for(int ii=0;ii<8;ii++)
			{
				tmp = GetNextNode(tmp);    		
				byte[0+ii]=tmp->data;     
			}  
    		
    		//do stuff with bits and stuff:
			
			if(byte[0])ibyte |=128;	
    		if(byte[1])ibyte |=64;	
    		if(byte[2])ibyte |=32;	
    		if(byte[3])ibyte |=16;
    		if(byte[4])ibyte |=8;	
    		if(byte[5])ibyte |=4;	
    		if(byte[6])ibyte |=2;	
    		if(byte[7])ibyte |=1;	
    
    		temp.append(1,(unsigned char)ibyte);
    	}
    
    	return temp;
    }
	
	
	string BitStream::ToHex(int groupnum,bool showx)
	{
        stringstream out;
		unsigned int ibyte;
		bool byte[8] = {0,0,0,0,0,0,0,0}; //used to store 8 bit chunks	
		int ii=groupnum;
		Node * tmp = NULL;
		
		if(groupnum<1 && showx)out<<"0x";
		
		for(int i=0;(i+8)<=m_bitsize;i+=8)
		{			
		
			if(ii>=groupnum && groupnum>0)
			{					
				ii=0;
				if(i>=8)out<<" ";
				if(showx)out<<"0x";	
			}
			ii++;
			

			ibyte=0;
						
			//get one byte:
			//-------------
			
			for(int biti=0;biti<8;biti++)
			{
				tmp = GetNextNode(tmp);    		
				
				byte[0+biti]=tmp->data; 
			}
			
			//This is backwards and is intentional
			if(byte[0])ibyte |=128;	
			if(byte[1])ibyte |=64;	
			if(byte[2])ibyte |=32;	
			if(byte[3])ibyte |=16;
			if(byte[4])ibyte |=8;	
			if(byte[5])ibyte |=4;	
			if(byte[6])ibyte |=2;	
			if(byte[7])ibyte |=1;

			if(ibyte<0xf)out<<"0";
			out<<hex<<uppercase<<ibyte;
		}
		
		
        out << hex <<uppercase;
        return out.str();
	}
	
	string BitStream::HexDump(unsigned int bytesperline)
	{		
		stringstream ss;
		
		string str = ToStr();
		
		int strsize=str.size();

		int pos=0;

		ss<<flush<<endl;

		while(pos<strsize)
		{
			for(int i=pos;i<pos+bytesperline;i++)
			{
				if(i>=strsize)break;
				ss<<flush<< setw(2) << setfill ( '0' )<<uppercase<< hex <<(unsigned int)(unsigned char)str[i]<<" ";
			}
			//if(slashrs)ss<<"\r";
			//ss<<"\n";						
			ss<<endl;
			
			for(int i=pos;i<pos+bytesperline;i++)
			{
				if(i>=strsize)break;
				
				if((int)str[i] < 32 || 126 < (int)str[i])ss<<".  ";
				else ss<< str[i]<<"  ";	
			}
		
			pos+=bytesperline;
			
			if(pos<strsize)
			{
				//if(slashrs)ss<<"\r\n\r\n";
				//else ss<<"\n\n";	
				ss<<endl<<endl;
			}
		}
		
		ss<<dec;
		
		return ss.str();	
	}

	
    
    string BitStream::ToBitStr(bool bytespaces=false)
    {	
		if(m_start==NULL)return "";
		
		string ret;
		Node * reading = m_start;
		
		int bitsshown=0;
				
		while(1)
		{
			if(bytespaces)
			{
				if(bitsshown==8)
				{
					ret.append(1,' ');
					bitsshown=0;
				}
				bitsshown++;
			}
			
			ret.append(1,reading->data ? '1' : '0');
			reading = GetNextNode(reading);
			if(reading==NULL)break;
		}		

        return ret;
    }
    

    

	
	
    void BitStream::DisplayInfo()
    {
		cout<<"First node: "<<(m_start!=NULL ? (m_start->data ? "1" : "0") : "NULL")<<"     Last node: "<<(m_end!=NULL ? (m_end->data ? "1" : "0") : "NULL")<<endl;
		cout<<"Stream size: "<<m_bitsize<<" bits"<<endl;		
    }
    
    void BitStream::Display()
    {
        cout<<endl<<HexDump()<<endl;
		DisplayInfo();
		cout<<endl;
    }
    
	
    void BitStream::Clear()
    {
		while(1)
		{
			Node * tmp=GetNode(0);
			if(tmp==NULL)return;
			DelNode(tmp);
		}
    }
    
    BitStream::bsint BitStream::Size()
    {
       return m_bitsize;
    }
    
    bool BitStream::IsEmpty()
    {
        return (m_bitsize==0);
    }
    

    
    void BitStream::DeleteBits(int todel,bsint pos)
    {
		Node * delref=GetNode(pos);
		Node * next=NULL;
		
		
		for(int i=0;i<todel;i++)
		{
			if(delref==NULL)break;
			next=delref->next;
			DelNode(delref);
			delref=next;
		}        
    }
	

	/*
	
	void BitStream::ReverseBits(int amt,bsint pos)
	{
		if(pos<0)pos=0;
		if((pos+amt)>Size())return;
		
		bool * tmp = new bool[amt];
		
		//write out stream to temp array:
		
		for(int i=0;i<amt;i++)
		{
			tmp[i]=	stream[pos+i];
		}
		
		//re-write it back to stream in reverse:
		
		int i2=amt-1;
		
		for(int i=0;i<amt;i++)
		{
			stream[pos+i]=tmp[(i2)];	
			i2--;
		}
	
		delete[] tmp;
	}
	
	
	//similar logic as reversebits, but does it in byte chunks	
	void BitStream::ReverseBytes(int amt,bsint pos)
	{
		if(pos<0)pos=0;
		if(((pos)+(amt*8))>Size())return;	
		
		
		
		BitStream::bsint * tmp = new BitStream::bsint[amt];
		
		//write out stream to temp array:
		
		for(int i=0;i<amt;i++)
		{
			tmp[i]=	ReadInt8(pos+(i*8));
		}
		
		//first clear what we got, then just re-add:
		
		DeleteBits(pos,amt*8);
		
		
		//re-write it back to stream in reverse:
		
		int i2=amt-1;
		
		for(int i=0;i<amt;i++)
		{			
			WriteInt8(tmp[i2],pos+(i*8));	
			i2--;
		}
	
		delete[] tmp;	
	
	}
			
       */ 
    
    //writers:
    //--------
    
	

    void BitStream::WriteBit(bool tw,bsint pos)
    {		
		WriteBit_internal(tw,GetNode(pos));
    }
    
    void BitStream::WriteVarInt(BitStream::bsint tw,int bits,bsint pos)
    {
		WriteVarInt_internal(tw,bits,GetNode(pos));
    }
    
    
    void BitStream::WriteInt8(BitStream::bsint tw,bsint pos)
    {
       WriteVarInt(tw,8,pos);
    }
    
    void BitStream::WriteInt16(BitStream::bsint tw,bsint pos)
    {
       WriteVarInt(tw,16,pos);
    }
    
    void BitStream::WriteInt24(BitStream::bsint tw,bsint pos)
    {
       WriteVarInt(tw,24,pos);
    }
    
    void BitStream::WriteInt32(BitStream::bsint tw,bsint pos)
    {
       WriteVarInt(tw,32,pos);
    }
    
    
    
    void BitStream::WriteString(string tw,bsint pos)
    {
        WriteString_internal(tw,GetNode(pos));
    }
    
    void BitStream::WriteBitStream(BitStream tw,bsint pos)
    {
        WriteBitStream_internal(tw,GetNode(pos));
    }
    

    
	void BitStream::WriteBit(bool towrite)
	{
		PushBit_e(towrite);
	}
	
	void BitStream::WriteVarInt(bsint towrite,int bitstouse)
	{
		PushIntVar_e(towrite,bitstouse);
	}
        
	void BitStream::WriteInt8(bsint towrite)
	{
		PushInt8_e(towrite);
	}

	void BitStream::WriteInt16(bsint towrite)
	{
		PushInt16_e(towrite);
	}

	void BitStream::WriteInt24(bsint towrite)
	{
		PushInt24_e(towrite);
	}

	void BitStream::WriteInt32(bsint towrite)
	{
		PushInt32_e(towrite);
	}

	void BitStream::WriteString(string towrite)
	{
		PushString_e(towrite);
	}

	void BitStream::WriteBitStream(BitStream towrite)
	{
		PushBitStream_e(towrite);
	}

    
    
    //readers
    //--------
    
    
    bool BitStream::ReadBit(bsint pos)
    {
        if(m_bitsize<=0)return false;
		
		Node * tmp = GetNode(pos);
		
		if(tmp==NULL)return false; //default to false
		
		return tmp->data;
    }
    
    

    BitStream::bsint BitStream::ReadVarInt(bsint pos,int bits)
    {
        return ReadVarInt_internal(bits,GetNode(pos));
    }
    
    BitStream::bsint BitStream::ReadInt8(bsint pos)
    {
       return ReadVarInt(pos,8);
    }
    
    BitStream::bsint BitStream::ReadInt16(bsint pos)
    {
       return ReadVarInt(pos,16);
    }
    
    BitStream::bsint BitStream::ReadInt24(bsint pos)
    {
      return ReadVarInt(pos,24);
    }
    
    BitStream::bsint BitStream::ReadInt32(bsint pos)
    {
       return ReadVarInt(pos,32);
    }
    
    
    string BitStream::ReadString(bsint pos,int chars)
    {
        if(m_bitsize<=0)return "";
    	
		return ReadString_internal(chars,GetNode(pos),false,false);
    }
    
    
    BitStream BitStream::ReadBitStream(bsint pos,int bits)
    {
        BitStream toreturn;
        if(m_bitsize<=0)return toreturn;
    
    	return ReadBitStream_internal(bits,GetNode(pos),false,false);
    }    
	
    
    
    //poppers
    //--------
    

    bool BitStream::PopBit_s()
    {
        if(m_bitsize<=0)return false;
		
		Node * tmpnode=m_start;

		if(m_start==NULL)return false; //default to 0
		
        bool pop=m_start->data;
		
		DelNode(m_start);       
        
        return pop;
    }
    
    bool BitStream::PopBit_e()
    {
        if(m_bitsize<=0)return false;
		
		Node * tmpnode=m_end;

		if(m_end==NULL)return false; //default to 0
		
        bool pop=m_end->data;
		
		DelNode(m_end);  
        
        return pop;            
    }


    BitStream::bsint BitStream::PopIntVar_s(int bits)
    {
        if(bits>MaxBits())bits=MaxBits();
        
        if(bits>m_bitsize)bits=m_bitsize;
        if(bits<=0)return 0;		
		
        return ReadVarInt_internal(bits,NULL,true,true);
    }
    
    BitStream::bsint BitStream::PopIntVar_e(int bits)
    {
        if(bits>MaxBits())bits=MaxBits();
        
        if(bits>m_bitsize)bits=m_bitsize-bits;
        if(bits<=0)return 0;
		        
        return ReadVarInt_internal(bits,NULL,false,true);
    }


    BitStream::bsint BitStream::PopInt8_s()
    {
        return PopIntVar_s(8);
    }
    
    BitStream::bsint BitStream::PopInt8_e()
    {
        return PopIntVar_e(8);
    }
    
    BitStream::bsint BitStream::PopInt16_s()
    {
        return PopIntVar_s(16);
    }
    
    BitStream::bsint BitStream::PopInt16_e()
    {
        return PopIntVar_e(16);
    }
    
    BitStream::bsint BitStream::PopInt24_s()
    {
        return PopIntVar_s(24);
    }
    
    BitStream::bsint BitStream::PopInt24_e()
    {
        return PopIntVar_e(24);
    }
    
    BitStream::bsint BitStream::PopInt32_s()
    {
        return PopIntVar_s(32);
    }
    
    BitStream::bsint BitStream::PopInt32_e()
    {
        return PopIntVar_e(32);
    }
    
    
    string BitStream::PopString_s(bsint chars)
    {
        if((chars*8)>m_bitsize)chars=(m_bitsize/8);
        if(chars<=0)return "";

        return ReadString_internal(chars,NULL,true,true);
    }
    
    
    string BitStream::PopString_e(bsint chars)
    {        
		if((chars*8)>m_bitsize)chars=(m_bitsize/8);
        if((chars*8)<=0)return "";
        
		return ReadString_internal(chars,NULL,false,true);
    }
    
    
    BitStream BitStream::PopBitStream_s(bsint bits)
    {
        BitStream ret;        
        if(bits>m_bitsize)bits=m_bitsize;
        if(bits<=0)return ret;        
        
		return ReadBitStream_internal(bits,NULL,true,true);
    }
    
    
    BitStream BitStream::PopBitStream_e(bsint bits)
    {        
        BitStream ret;  
        if(bits>m_bitsize)bits=m_bitsize;
        if(bits<=0)return ret;
        
		return ReadBitStream_internal(bits,NULL,false,true);      
    } 
    
    
    //pushers
    //-------
    
    void BitStream::PushBit_s(bool tw)
    {
        Node * added = AddNode(true);
		added->data=tw;
    }
    
    void BitStream::PushBit_e(bool tw)
    {
        Node * added = AddNode(false);
		added->data=tw;
    }
    
    void BitStream::PushIntVar_s(BitStream::bsint tw,int bits)
    {
        WriteVarInt_internal(tw,bits,NULL,true);
    }
    
    void BitStream::PushIntVar_e(BitStream::bsint tw,int bits)
    {
        WriteVarInt_internal(tw,bits,NULL,false);
    }
    
    void BitStream::PushInt8_s(BitStream::bsint  tw)
    {
        PushIntVar_s(tw,8);
    }
    
    void BitStream::PushInt8_e(BitStream::bsint  tw)
    {
        PushIntVar_e(tw,8);
    }
    
    void BitStream::PushInt16_s(BitStream::bsint  tw)
    {
        PushIntVar_s(tw,16);
    }
    
    void BitStream::PushInt16_e(BitStream::bsint  tw)
    {
        PushIntVar_e(tw,16);
    }
    
    void BitStream::PushInt24_s(BitStream::bsint  tw)
    {
        PushIntVar_s(tw,24);
    }
    
    void BitStream::PushInt24_e(BitStream::bsint  tw)
    {
        PushIntVar_e(tw,24);
    }
    
    void BitStream::PushInt32_s(BitStream::bsint  tw)
    {
        PushIntVar_s(tw,32);
    }
    
    void BitStream::PushInt32_e(BitStream::bsint  tw)
    {
        PushIntVar_e(tw,32);
    }
    
	void BitStream::PushString_s(string tw)
	{
		WriteString_internal(tw,NULL,true);
	}
	
	void BitStream::PushString_e(string tw)
	{
		WriteString_internal(tw,NULL,false);
	}
	
	void BitStream::PushBitStream_s(BitStream tw)
	{
		WriteBitStream_internal(tw,NULL,true);
	}
	
	void BitStream::PushBitStream_e(BitStream tw)
	{
		WriteBitStream_internal(tw,NULL,false);
	}
    	
    
    
    //overloaders
    //-----------

    
    BitStream &BitStream::operator=(BitStream& tw)
    {
    	if (this == &tw) return *this;    //make sure we're not assigning ourselves to ourselves
    
    	Clear();
    	PushBitStream_s(tw);
    	return *this;
    }
	
    BitStream BitStream::operator=(const BitStream& tw)
    {
    	if (this == &tw) return *this;    //make sure we're not assigning ourselves to ourselves
    
    	Clear();
    	PushBitStream_s(tw);
    	return *this;
    }	
    
    BitStream& BitStream::operator=(string& tw)
    {    
    	Clear();
    	PushString_s(tw);
    	return *this;
    }  
	
    
    BitStream BitStream::operator=(const string& tw)
    {    
    	Clear();
    	PushString_s(tw);
    	return *this;
    }  	

	
    
    BitStream& BitStream::operator+=(BitStream& tw)
    {
    	PushBitStream_e(tw);
    	return *this;
    }
        
    BitStream& BitStream::operator+=(string tw)
    {
    	PushString_e(tw);
    	return *this;
    }
    

	

	BitStream BitStream::operator+(const BitStream &other) const
	{
		BitStream ret;
		ret.PushBitStream_e(*this);
		ret.PushBitStream_e(other);
		return ret;
	}
	
	BitStream BitStream::operator+(const string &other) const
	{
		BitStream ret;
		ret.PushBitStream_e(*this);
		ret.PushString_e(other);
		return ret;
	}
	
	BitStream BitStream::operator+(const char*& other) const
	{
		BitStream ret;
		ret.PushBitStream_e(*this);
		ret.PushString_e(string(other));
		return ret;
	}	

    
    
    bool BitStream::operator==( BitStream & tw)
    {
        return (ToBitStr()==tw.ToBitStr());
    }
    
    bool BitStream::operator==( string tw)
    {
        return (ToStr()==tw);
    }    
    
    
    
    
    //statics/utility
    //---------------
	
	
	
    
    int BitStream::MaxBits()
    { 
        return MaxBitsOf(ULONG_MAX);
    }
    
    int BitStream::MaxBitsOf(BitStream::bsint a)
    { 
        return (int)ceil(log(a)/log(2));
    }    
    
    deque<bool> BitStream::Int2Bin(BitStream::bsint tc)
    {
    	BitStream::bsint ander=1;
    	
    	deque<bool> ret;
    	
    	while(ander<=tc && ander>0)
    	{
            if(tc & ander) ret.push_front(true);            
            else ret.push_front(false);   
            ander*=2;
        }
        
        if(ret.size()<=0)ret.push_front(false);
               
        return ret;
    }
    
    
    string BitStream::Int2BinStr(BitStream::bsint tc)
    {
        deque<bool> tmp = Int2Bin(tc);
        
        string ret="";
        
        for(int i=0;i<tmp.size();i++) ret+= tmp[i]? "1" : "0";
        
        return ret;
    } 
    
    BitStream BitStream::Int2BinBS(BitStream::bsint tc)
    {
        deque<bool> tmp = Int2Bin(tc);
        
        BitStream ret;
        
        for(int i=0;i<tmp.size();i++) ret.PushBit_e(tmp[i]);
        
        return ret;
    }    
    
    string BitStream::Int2Str(BitStream::bsint tc)
    {
        stringstream out;
        out << tc;
        return out.str();
    }   
	
    string BitStream::Int2HexStr(BitStream::bsint tc,bool dontshowx=false)
    {
        stringstream out;
		
		if(!dontshowx)out<<"0x";		
        out << hex <<uppercase<<tc;
        return out.str();
    }   	
	
	string BitStream::Str2HexStr(string tc,int groupnum=1,bool showx=true)
	{
		stringstream out;

		int ii=groupnum;
		
		if(groupnum<1 && showx)out<<"0x";
		
		for(int i=0;i<tc.size();i++)
		{			
		
			if(ii>=groupnum && groupnum>0)
			{					
				ii=0;
				if(i>0)out<<" ";
				if(showx)out<<"0x";	
			}
			ii++;
			
			out<<hex<<uppercase<<(unsigned int)(unsigned char)tc[i];			
		}
		

       return out.str();
	}
	
    string BitStream::Double2Str(long double tc)
	{
        stringstream out;
        out << tc;
        return out.str();
	}

	BitStream::bsint BitStream::Str2Int(string tc)
	{
		return atol(tc.c_str());
	}	
	
	
}


