/*
======================================================
rslib component:  ByteStream implimentors
======================================================
*/


namespace rslib
{

	ByteStream::Node::Node()
	{
		prev=NULL;
		next=NULL;
	}
	
	ByteStream::Node::Node(Node * prevN,Node * nextN)
	{
		prev=prevN;
		next=nextN;
		
		if(prevN!=NULL)prevN->next=this;
		if(nextN!=NULL)nextN->prev=this;		
	}


	ByteStream::ByteStream()
    {
		m_size=0;		
		m_start=NULL;
		m_end=NULL;
    }
	
	ByteStream::ByteStream(const ByteStream &copy)
	{
		m_size=0;		
		m_start=NULL;
		m_end=NULL;
		
		Node * reading = copy.m_start;
				
		while(reading!=NULL)
		{
			Node * toadd = AddNode();
			toadd->data=reading->data;
			
			reading = GetNextNode(reading);
			if(reading==NULL)break;
		}
	}
	
	
	ByteStream::ByteStream(const string &copy)
	{
		m_size=0;		
		m_start=NULL;
		m_end=NULL;
		
		WriteString(copy);
	}	
	
	ByteStream::~ByteStream()
	{
		Clear();
	}
	

	bool ByteStream::EnsureMaxIntSize(uli & tw,int bytes)
	{
		uli mask=0xff;		
		uli maxtw=0xff;
		
		for(int i=1;i<bytes;i++)
		{					
			maxtw = maxtw << 8;
			maxtw|=0xff;
		}
		
		if(tw>maxtw)
		{
			tw=maxtw;
			return false;
		}
		return true;
	}
    
	bool ByteStream::EnsureMaxPos(uli & pos,uli & bytes)
	{
		if(bytes>m_size)
		{
			pos=0;
			bytes=m_size;
			return false;
		}		
		if((pos+bytes)>m_size-1)
		{
			bytes=m_size-pos;
			return false;
		}
		return true;
	}		
	
	
	ByteStream::Node * ByteStream::AddNode(bool start)
	{
		Node * newnode=NULL;		
		
		if(m_start==NULL) 		//this is the first node ever added
		{			
			m_size++;
			newnode = new Node();
			m_start=newnode;
			m_end=newnode;
		}		
		else if(start)  			//we want to add the node at the start
		{			
			m_size++;
			newnode = new Node(NULL,m_start);
			m_start=newnode;			
		}		
		else					//we want to add at the end
		{			
			m_size++;
			newnode = new Node(m_end,NULL);
			m_end=newnode;		
		}
		return newnode;
	}
	
	ByteStream::Node * ByteStream::InsertNodeAfter(Node * ref)
	{
		Node * newnode = NULL;
		
		if(m_start==NULL)			//if start is null, then this is the first node ever
		{
			newnode = AddNode();
		}
		else if(ref==NULL)			//if ref is null, assume we want to add at the start
		{
			newnode = AddNode(true);
		}
		else if(ref->next==NULL)  	//if "next" is null then it means ref is the last node, new node is now last
		{
			m_size++;
			newnode = new Node(ref,NULL);
			m_end=newnode;	
		}
		else							//standard insert after ref node
		{
			m_size++;
			newnode = new Node(ref,ref->next);
		}
		return newnode;
	}
	
	ByteStream::Node * ByteStream::InsertNodeBefore(Node * ref)
	{
		Node * newnode = NULL;
		
		if(m_start==NULL)			//if start is null, then this is the first node ever
		{
			newnode = AddNode();
		}
		else if(ref==NULL)			//if ref is null, assume we want to add at the start
		{
			newnode = AddNode(true);
		}
		else if(ref->prev==NULL)  	//if "prev" is null then it means ref is the first node, new node is now first
		{
			m_size++;
			newnode = new Node(NULL,ref);
			m_start=newnode;	
		}
		else							//standard insert after ref node
		{
			m_size++;
			newnode = new Node(ref->prev,ref);
		}
		return newnode;
	}	
	
	void ByteStream::DelNode(ByteStream::Node * ref)
	{
		if(m_size<=0 || ref==NULL)return;
		
		
		if(ref->prev==NULL && ref->next==NULL)  //this node is both the start and the end (aka, only one node in the system)
		{
			m_start=NULL;
			m_end=NULL;
		}
		else if(ref->prev==NULL)	//this node is the start, and there are other nodes after
		{
			m_start=ref->next;
			ref->next->prev=NULL;		
		}
		else if(ref->next==NULL) //this is the last node
		{
			m_end=ref->prev;
			ref->prev->next=NULL;
		}
		else //this node is in the middle
		{
			ref->prev->next = ref->next;
			ref->next->prev = ref->prev;
		}
		m_size--;
		delete ref;
	}
    
	ByteStream::Node * ByteStream::GetNextNode(ByteStream::Node * ref)
	{
		if(ref==NULL)return m_start;			//if argument is null, then we are starting from the beginning		
		else if(ref==m_end)return NULL; 		//if we're at the end, just return null
		
		return ref->next;
	}
	
	ByteStream::Node * ByteStream::GetPrevNode(ByteStream::Node * ref)
	{
		if(ref==NULL)return m_end;			//if argument is null, then we are starting from the end	
		else if(ref==m_start)return NULL; 		//if we're at the start, just return null
		
		return ref->prev;
	}	
	
	ByteStream::Node * ByteStream::GetNode(bsint pos)
	{
		if(pos>=m_size)return NULL; //if position is past end we return null
		
		Node * ret=NULL;
		
		if(pos==0)return m_start; //if pos is 0 then we want start
		else if(pos==m_size-1)return m_end; //if pos is size-1 then we want end
		
		//we want a node in middle of list:
		
		//determine what is faster, seeking from start to node or from end to node
		
		if(pos>(m_size/2))	//seek from end to node
		{
			ret=m_end;
			for(bsint i=m_size-1;i!=pos;i--)
			{				
				ret = GetPrevNode(ret);
			}
		}
		else	//seek from start to node
		{		
			ret=m_start;
			for(bsint i=0;i<pos;i++)
			{
				ret = GetNextNode(ret);
			}
		}
		
		return ret;
	}	
	
	
	void ByteStream::WriteVarInt_internal(uli tw,int bytes, Node * ref, bool start)
	{
		if(bytes==0)bytes=1;
		else if(bytes>MaxBytes())bytes=MaxBytes();
		
		EnsureMaxIntSize(tw,bytes);
		
		uli twtmp=tw;		
		uli mask=0xff; //ANDing mask, shifted for each byte we write
		
		Node * addnode=NULL;	
		
		//bytes are written (in reverse, so always one before the last one)
		for(int i=0;i<bytes;i++)
		{
			if(addnode==NULL) //if this is the first node of the bs we are adding, do some priliminary checking/setting
			{
				if(ref==NULL) addnode = AddNode(start); //add node at start or end					
				else addnode = InsertNodeBefore(ref);
			}
			else addnode = InsertNodeBefore(addnode);
			
			addnode->data=(unsigned char)((twtmp & mask)>>8*i);
			
			mask = mask << 8; //shift mask over so we can do next byte
		}
	}

	void ByteStream::WriteString_internal(string tw,Node * ref,bool start)
	{		
		Node * addnode=NULL;		
		
		for(uli i=0;i<tw.size();i++)
		{
			if(addnode==NULL) //if this is the first node of the bs we are adding, do some priliminary checking/setting
			{
				if(ref==NULL)addnode = AddNode(start); //add node at start or end	
				else addnode = InsertNodeBefore(ref);
			}
			else addnode = InsertNodeAfter(addnode);
					
			addnode->data = tw[i];
		}
		
	}
	
	
	
	void ByteStream::WriteByteStream_internal(ByteStream tw,Node * ref,bool start)
	{
		
		Node * twnode=NULL;
		Node * addnode=NULL;
		
		while(1)
		{
			twnode = tw.GetNextNode(twnode);
			if(twnode==NULL)break;
			
			
			if(addnode==NULL) //if this is the first node of the bs we are adding, do some priliminary checking/setting
			{
				if(ref==NULL)addnode = AddNode(start); //add node at start or end	
				else addnode = InsertNodeBefore(ref);
			}
			else addnode = InsertNodeAfter(addnode);			//add node using last added node 
			
			addnode->data=twnode->data;
		}	
	}

	
	uli ByteStream::ReadVarInt_internal(int bytes,Node * ref,bool start,bool pop)
	{		
		uli ret=0;    
		Node * tmpnode=NULL;
    			
		
		if(ref==NULL)
		{
			if(start || ((bytes)>=m_size))ref=m_start;
			else ref = GetNode(m_size-bytes);
			
			//there's still a chance for ref to be null here so we check again and return nothing if it is.
			if(ref==NULL)return 0;
		}
		
		Node * tmp = ref; 
    
		int ii=bytes;
    	for(int i=0;i<bytes;i++)
    	{			
			ii--;			

			ret|= (uli)(tmp->data<<(ii*8));
			
			tmp = GetNextNode(tmp);			
			if(tmp==NULL)break;			
    	}
		
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<bytes;i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}	
		
    
    	return ret;
	}	
	
	string ByteStream::ReadString_internal(bsint chars,Node * ref,bool start,bool pop)
	{
		string ret;    
		Node * tmpnode=NULL;
    			
		
		if(ref==NULL)
		{
			if(start || ((chars)>=m_size))ref=m_start;
			else ref = GetNode(m_size-chars);
			
			//there's still a chance for ref to be null here so we check again and return nothing if it is.
			if(ref==NULL)return "";
		}
		
		Node * tmp = ref; 
    
    	for(uli i=0;i<chars;i++)
    	{			
			ret.append(1,tmp->data);
			
			tmp = GetNextNode(tmp);
			
			if(tmp==NULL)break;			
    	}
		
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<chars;i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}	
		
    
    	return ret;
	}
	
	
	
	ByteStream ByteStream::ReadByteStream_internal(bsint bytes,Node * ref,bool start,bool pop)
	{
		
		ByteStream ret;    
		Node * tmpnode=NULL;
		
		if(ref==NULL)
		{
			if(start || (bytes>=m_size))ref=m_start;
			else ref = GetNode(m_size-bytes);
			
			//there's still a chance for ref to be null here so we check again and return nothing if it is.
			if(ref==NULL)return ret;
		}
		
		Node * tmp = ref; 
    
    	for(int i=0;i<bytes;i++)
    	{         			 
			tmpnode = ret.AddNode();
			tmpnode->data=tmp->data;
			
			tmp = GetNextNode(tmp);			
    	}
		
		
		
		if(pop)
		{
			tmpnode = ref;
			Node * todel=tmpnode;
			for(int i=0;i<bytes;i++)
			{
				tmpnode=todel->next;
				DelNode(todel);
				todel=tmpnode;
			}
		}	
		
    
    	return ret;
	}
	
	
	

	bool ByteStream::Lock(bool block=true)
	{
		#ifdef rslib_pthread
		if(block)mutex.Lock();
		else return mutex.TryLock();
		#endif

		return true;
	}

	
	void ByteStream::Unlock()
	{
		#ifdef rslib_pthread
		mutex.UnLock();
		#endif
	}
	
	

    string ByteStream::ToStr()
    {
		return ReadString_internal(m_size,NULL,true,false);
    }
	
	
	string ByteStream::ToHex(int groupnum,bool showx)
	{        
		stringstream out;

		int ii=groupnum;
		Node * tmp = NULL;
		unsigned char byte;
		
		if(groupnum<1 && showx)out<<"0x";
		
		for(int i=0;i<m_size;i++)
		{
		
			if(ii>=groupnum && groupnum>0)
			{					
				ii=0;
				if(i>=1)out<<" ";
				if(showx)out<<"0x";	
			}
			ii++;
			

						
			//get one byte:
			//-------------
			tmp = GetNextNode(tmp); 
			byte = tmp->data;

			if(byte<0xf)out<<"0";
			out<<hex<<uppercase<<(unsigned int)byte;
		}
		
		
        out << hex <<uppercase;
        return out.str();
	}
	
	string ByteStream::HexDump(unsigned int bytesperline)
	{		
		stringstream ss;
		
		string str = ToStr();
		
		int strsize=str.size();

		int pos=0;

		ss<<flush<<endl;

		while(pos<strsize)
		{
			for(int i=pos;i<pos+bytesperline;i++)
			{
				if(i>=strsize)break;
				ss<<flush<< setw(2) << setfill ( '0' )<<uppercase<< hex <<(unsigned int)(unsigned char)str[i]<<" ";
			}						
			ss<<endl;
			
			for(int i=pos;i<pos+bytesperline;i++)
			{
				if(i>=strsize)break;
				
				if((int)str[i] < 32 || 126 < (int)str[i])ss<<".  ";
				else ss<< str[i]<<"  ";	
			}
		
			pos+=bytesperline;
			
			if(pos<strsize)
			{	
				ss<<endl<<endl;
			}
		}
		
		ss<<dec;
		
		return ss.str();
	}



    

	
	
    void ByteStream::DisplayInfo()
    {
		stringstream ss;
		
		ss<<"First Node: ";		
		if(m_start!=NULL)ss<<"0x"<<flush<<setw(2) << setfill ( '0' )<<uppercase<<hex<<(unsigned int)(unsigned char)m_start->data;
		else ss<<"NULL";
		
		ss<<endl<<"Last Node: ";		
		if(m_end!=NULL)ss<<"0x"<<flush<<setw(2) << setfill ( '0' )<<uppercase<<hex<<(unsigned int)(unsigned char)m_end->data;
		else ss<<"NULL";
		
		
		cout<<ss.str()<<endl;
		cout<<"Stream size: "<<m_size<<" bytes"<<endl;		
    }
    
    void ByteStream::Display()
    {		
		cout<<endl<<HexDump()<<endl<<"----"<<endl;
		DisplayInfo();
		cout<<endl;
    }
    
	
    void ByteStream::Clear()
    {
		while(1)
		{
			Node * tmp=GetNode(0);
			if(tmp==NULL)return;
			DelNode(tmp);
		}
    }
    
    ByteStream::bsint ByteStream::Size()
    {
       return m_size;
    }
    
    bool ByteStream::IsEmpty()
    {
        return (m_size==0);
    }
    


		
       
    
    //writers:
    //--------
    
	

    
    void ByteStream::WriteInt8(ByteStream::bsint tw,bsint pos)
    {
       WriteVarInt_internal(tw,1, GetNode(pos),false);
    }
    
    void ByteStream::WriteInt16(ByteStream::bsint tw,bsint pos)
    {
       WriteVarInt_internal(tw,2, GetNode(pos),false);
    }
    
    void ByteStream::WriteInt24(ByteStream::bsint tw,bsint pos)
    {
      WriteVarInt_internal(tw,3, GetNode(pos),false);
    }
    
    void ByteStream::WriteInt32(ByteStream::bsint tw,bsint pos)
    {
       WriteVarInt_internal(tw,4, GetNode(pos),false);
    }
    
    
    
    void ByteStream::WriteString(string tw,bsint pos)
    {		
		WriteString_internal(tw,GetNode(pos)); 
    }
    
    void ByteStream::WriteByteStream(ByteStream tw,bsint pos)
    {
		WriteByteStream_internal(tw,GetNode(pos)); 		
    }
    


	void ByteStream::WriteInt8(bsint tw)
	{
		WriteVarInt_internal(tw,1, NULL,false);
	}

	void ByteStream::WriteInt16(bsint tw)
	{
		WriteVarInt_internal(tw,2, NULL,false);
	}

	void ByteStream::WriteInt24(bsint tw)
	{
		WriteVarInt_internal(tw,3, NULL,false);
	}

	void ByteStream::WriteInt32(bsint tw)
	{
		WriteVarInt_internal(tw,4, NULL,false);
	}

	void ByteStream::WriteString(string tw)
	{
		WriteString_internal(tw,NULL,false); 
	}

	void ByteStream::WriteByteStream(ByteStream tw)
	{
		WriteByteStream_internal(tw,NULL,false);
	}

    
    
    //readers
    //--------
    

	ByteStream::bsint ByteStream::ReadVarInt(bsint pos,int bytes)
	{
	   uli bytesuli = bytes;
	   if(bytesuli>MaxBytes())bytesuli=MaxBytes();	   
	   EnsureMaxPos(pos,bytesuli);
	   
	   return ReadVarInt_internal(bytesuli,GetNode(pos),false,false);
	}
	

    
    ByteStream::bsint ByteStream::ReadInt8(bsint pos)
    {
		ReadVarInt(pos,1);
    }
    
    ByteStream::bsint ByteStream::ReadInt16(bsint pos)
    {
       ReadVarInt(pos,2);
    }
    
    ByteStream::bsint ByteStream::ReadInt24(bsint pos)
    {
		ReadVarInt(pos,3);
    }
    
    ByteStream::bsint ByteStream::ReadInt32(bsint pos)
    {
      ReadVarInt(pos,4);
    }
    
    
    string ByteStream::ReadString(bsint pos,bsint chars)
    {       
		EnsureMaxPos(pos,chars);
    	
		return ReadString_internal(chars,GetNode(pos),false,false);		
    }
    
    
    ByteStream ByteStream::ReadByteStream(bsint pos,bsint bytes)
    {
		EnsureMaxPos(pos,bytes);
    
		return ReadByteStream_internal(bytes,GetNode(pos),false,false);
    }    
	
    
    
    //poppers
    //--------


    ByteStream::bsint ByteStream::PopInt8_s()
    {
       return ReadVarInt_internal(1,NULL,true,true);
    }
    
    ByteStream::bsint ByteStream::PopInt8_e()
    {
       return ReadVarInt_internal(1,NULL,false,true);
    }
    
    ByteStream::bsint ByteStream::PopInt16_s()
    {
       return ReadVarInt_internal(2,NULL,true,true);
    }
    
    ByteStream::bsint ByteStream::PopInt16_e()
    {
       return ReadVarInt_internal(2,NULL,false,true);
    }
    
    ByteStream::bsint ByteStream::PopInt24_s()
    {
        return ReadVarInt_internal(3,NULL,true,true);
    }
    
    ByteStream::bsint ByteStream::PopInt24_e()
    {
       return ReadVarInt_internal(3,NULL,false,true);
    }
    
    ByteStream::bsint ByteStream::PopInt32_s()
    {
       return ReadVarInt_internal(4,NULL,true,true);
    }
    
    ByteStream::bsint ByteStream::PopInt32_e()
    {
       return ReadVarInt_internal(4,NULL,false,true);
    }
    
    
    string ByteStream::PopString_s(bsint chars)
    {
		if((chars*8)>m_size)chars=(m_size);
		if(chars<=0)return "";

        return ReadString_internal(chars,NULL,true,true);
    }
    
    
    string ByteStream::PopString_e(bsint chars)
    {		
		if((chars)>m_size)chars=(m_size);
        if((chars)<=0)return "";
        
		return ReadString_internal(chars,NULL,false,true);
    }
    
    
    ByteStream ByteStream::PopByteStream_s(bsint bits)
    {
       
		ByteStream ret;        
        if(bits>m_size)bits=m_size;
        if(bits<=0)return ret;        
        
		return ReadByteStream_internal(bits,NULL,true,true);
    }
    
    
    ByteStream ByteStream::PopByteStream_e(bsint bits)
    {    
		ByteStream ret;  
        if(bits>m_size)bits=m_size;
        if(bits<=0)return ret;
        
		return ReadByteStream_internal(bits,NULL,false,true);      
    } 
    
    
    //pushers
    //-------

    

    
    void ByteStream::PushInt8_s(ByteStream::bsint  tw)
    {
       WriteVarInt_internal(tw,1,NULL,true);
    }
    
    void ByteStream::PushInt8_e(ByteStream::bsint  tw)
    {
       WriteVarInt_internal(tw,1,NULL,false);
    }
    
    void ByteStream::PushInt16_s(ByteStream::bsint  tw)
    {
		WriteVarInt_internal(tw,2,NULL,true);
    }
    
    void ByteStream::PushInt16_e(ByteStream::bsint  tw)
    {
       WriteVarInt_internal(tw,2,NULL,false);
    }
    
    void ByteStream::PushInt24_s(ByteStream::bsint  tw)
    {
       WriteVarInt_internal(tw,3,NULL,true);
    }
    
    void ByteStream::PushInt24_e(ByteStream::bsint  tw)
    {
        WriteVarInt_internal(tw,3,NULL,false);
    }
    
    void ByteStream::PushInt32_s(ByteStream::bsint  tw)
    {
        WriteVarInt_internal(tw,4,NULL,true);
    }
    
    void ByteStream::PushInt32_e(ByteStream::bsint  tw)
    {
       WriteVarInt_internal(tw,4,NULL,false);
    }
    
	void ByteStream::PushString_s(string tw)
	{
		WriteString_internal(tw,NULL,true);
	}
	
	void ByteStream::PushString_e(string tw)
	{
		WriteString_internal(tw,NULL,false);
	}
	
	void ByteStream::PushByteStream_s(ByteStream tw)
	{
		WriteByteStream_internal(tw,NULL,true);
	}
	
	void ByteStream::PushByteStream_e(ByteStream tw)
	{
		WriteByteStream_internal(tw,NULL,false);
	}
    	
    
	//statics:
	//-------
	
	int ByteStream::MaxBytes()
	{
		return 4; //TODO: make this dynamic based on cpu and stuff
	}
	

	
    //overloaders
    //-----------

    
    ByteStream &ByteStream::operator=(ByteStream& tw)
    {
    	if (this == &tw) return *this;    //make sure we're not assigning ourselves to ourselves
    
    	Clear();
    	PushByteStream_s(tw);
    	return *this;
    }
	
    ByteStream ByteStream::operator=(const ByteStream& tw)
    {
    	if (this == &tw) return *this;    //make sure we're not assigning ourselves to ourselves
    
    	Clear();
    	PushByteStream_s(tw);
    	return *this;
    }	
		
    
    ByteStream& ByteStream::operator=(string& tw)
    {    
    	Clear();
    	PushString_s(tw);
    	return *this;
    }  
	
    
    ByteStream ByteStream::operator=(const string& tw)
    {    
    	Clear();
    	PushString_s(tw);
    	return *this;
    }  	

	
    
    ByteStream& ByteStream::operator+=(ByteStream& tw)
    {
    	PushByteStream_e(tw);
    	return *this;
    }
        
    ByteStream& ByteStream::operator+=(string tw)
    {
    	PushString_e(tw);
    	return *this;
    }
    

	

	ByteStream ByteStream::operator+(const ByteStream &other) const
	{
		ByteStream ret;
		ret.PushByteStream_e(*this);
		ret.PushByteStream_e(other);
		return ret;
	}
	
	ByteStream ByteStream::operator+(const string &other) const
	{
		ByteStream ret;
		ret.PushByteStream_e(*this);
		ret.PushString_e(other);
		return ret;
	}
	
	ByteStream ByteStream::operator+(const char*& other) const
	{
		ByteStream ret;
		ret.PushByteStream_e(*this);
		ret.PushString_e(string(other));
		return ret;
	}	

    
    
    bool ByteStream::operator==( ByteStream & tw)
    {
        return (ToStr()==tw.ToStr());
    }
    
    bool ByteStream::operator==( string tw)
    {
        return (ToStr()==tw);
    }    
    
    

	
}


