/*
======================================================
rslib component: Containers prototypes
======================================================
*/

#ifndef containers_h
#define containers_h




namespace rslib
{
	//extended vector class - features thread safety and extra functionability
	template <class T>
	class xVector
	{
		private:
		#ifdef rslib_pthread
			MUTEX internalmutex;
			MUTEX externalmutex;
		#endif
		
		void InternalLock();
		void InternalUnlock();
		
		public:
		vector<T> DATA; //we make this public in case we need to access it directly		
		bool ThreadSafe;//turn off to disable thread safety (for less processing) - disabled by default if not using pthread
		
		xVector();
		~xVector();
		
		void Lock(); 	//lock for i/o  (multithreading)
		bool TryLock(); 	//lock for i/o  (multithreading)
		void Unlock();			//unlock for i/o(multithreading)
		bool IsEmpty();			//check if its empty
		
		unsigned long int GetSize();		//get current size
		unsigned long int GetMaxsize();		//get max system size
		
		bool Erase(T);				//erase by element
		bool EraseAt(int pos);		//erase by position  - unsafe used internally only (thread unsafe)
		bool EraseAt(int pos,bool unsafe);		//erase by position
		bool PopFrom(T &give,int pos);			//pop item from given position
		bool Get(T &give,int pos);	//get item by reference, returns false if index is invalid or no items to give
		bool Find(T tofind);		//check if an item exists
		void Add(T item,int pos);	//add item 
		void Add(T item);	//add item 
		void Clear();				//clear array
		
		xVector<T> & operator+=(T tw);//add element using +=
	};

}


#endif

