/*
======================================================
rslib component: date/time implimentors
======================================================
*/


namespace rslib
{

	DateTime::DateTime()
	{
		m_time=0;
		RebuildTS();
		m_format="M-d-Y h:i:sa";
	}
		
	DateTime::DateTime(time_t setto)
	{
		m_time=setto;
		RebuildTS();
		m_format="M-d-Y h:i:sa";		
	}
	
	void DateTime::RebuildTime()
	{
		#ifdef rslib_pthread		
			while(!tmmutex.TryLock())Wait(1);		
		#endif		
		
		struct tm * timeinfo;
		
		time(&m_time);
		timeinfo = localtime(&m_time);
		
		timeinfo->tm_year = tm_year;
		timeinfo->tm_mon = tm_mon;
		timeinfo->tm_mday = tm_mday;
		
		timeinfo->tm_hour=tm_hour;
		timeinfo->tm_min=tm_min;
		timeinfo->tm_sec=tm_sec;		
		
		m_time = mktime(timeinfo);

		#ifdef rslib_pthread		
			tmmutex.UnLock();	
		#endif
	}
	
	void DateTime::RebuildTS()
	{
		#ifdef rslib_pthread		
			while(!tmmutex.TryLock())Wait(1);		
		#endif
		
		struct tm * ptm;
		ptm = localtime( &m_time );

		tm_sec=ptm->tm_sec;
		tm_min=ptm->tm_min;
		tm_hour=ptm->tm_hour;
		tm_mday=ptm->tm_mday;
		tm_mon=ptm->tm_mon;
		tm_year=ptm->tm_year;
		tm_wday=ptm->tm_wday;
		tm_yday=ptm->tm_yday;
		tm_isdst=ptm->tm_isdst;

		#ifdef rslib_pthread		
			tmmutex.UnLock();	
		#endif		
	}
	
	
	void DateTime::SetFormat(string format)
	{
		m_format=format;
	}
	
	string DateTime::GetDate(string format)
	{
		if(m_time==0)return "[never]";
		
		
		
		string ret;
					
		for(int i=0;i<format.size();i++)
		{
			switch(format[i])
			{
				case 'd':ret+=GDF_d(tm_mday); break;	
				case 'D':ret+=GDF_D(tm_wday); break;	
				case 'j':ret+=GDF_j(tm_mday); break;	
				case 'l':ret+=GDF_l(tm_wday); break;	
				case 'N':ret+=GDF_N(tm_wday); break;	
				case 'S':ret+=GDF_S(tm_mday); break;	
				case 'w':ret+=GDF_w(tm_wday); break;	
				case 'z':ret+=GDF_z(tm_yday); break;
				case 'F':ret+=GDF_F(tm_mon); break;	
				case 'm':ret+=GDF_m(tm_mon); break;	
				case 'M':ret+=GDF_M(tm_mon); break;	
				case 'n':ret+=GDF_n(tm_mon); break;	
				case 't':ret+=GDF_t(tm_year,tm_mon); break;	
				case 'L':ret+=GDF_L(tm_year); break;	
				case 'Y':ret+=GDF_Y(tm_year); break;	
				case 'y':ret+=GDF_y(tm_year); break;	
				case 'a':ret+=GDF_a(tm_hour); break;	
				case 'A':ret+=GDF_A(tm_hour); break;	
				case 'g':ret+=GDF_g(tm_hour); break;	
				case 'G':ret+=GDF_G(tm_hour); break;	
				case 'h':ret+=GDF_h(tm_hour); break;	
				case 'H':ret+=GDF_H(tm_hour); break;	
				case 'i':ret+=GDF_i(tm_min); break;	
				case 's':ret+=GDF_s(tm_sec); break;	
				case 'I':ret+=GDF_I(tm_isdst); break;	
				case 'U':ret+=GDF_U(m_time); break;	
				default: ret+=format.substr(i,1);
			}		
		}
		
	
		return ret;
	}
	
		
	string DateTime::GetDate()
	{
		return GetDate(m_format);
	}	
	
	
	void DateTime::SetYear(int year)
	{
		tm_year = (year-1900);	
		RebuildTime();
	}
	
	void DateTime::SetMonth(int month)
	{
		if(month<1)month=1;
		if(month>12)month=12;
		month--;
		
		tm_mon=month;	
		RebuildTime();
	}
	
	void DateTime::SetDay(int day)
	{
		int maxdays=atoi(GDF_t(tm_year,tm_mon).c_str());
		
		if(day<1)day=1;
		if(day>maxdays)day=maxdays;
		
		tm_mday=day;
		RebuildTime();
	}
	
	void DateTime::SetHour(int hour)
	{
		if(hour<0)hour=0;
		if(hour>23)hour=23;
	
		tm_hour=hour;		
		RebuildTime();
	}
	
	void DateTime::SetMinute(int minute)
	{
		if(minute<0)minute=0;
		if(minute>59)minute=59;
		
		tm_min=minute;		
		RebuildTime();
	}
	
	void DateTime::SetSecond(int second)
	{
		if(second<0)second=0;
		if(second>59)second=59;
		
		tm_sec=second;		
		RebuildTime();
	}

	
	
	void DateTime::AddSeconds(int secs)
	{
		m_time+=secs;
		RebuildTS();
	}
	
	void DateTime::AddMinutes(int mins)
	{
		m_time+=mins*60;
		RebuildTS();
	}
	
	void DateTime::AddHours(int hours)
	{
		m_time+=hours*3600;
		RebuildTS();
	}
	
	void DateTime::AddDays(int days)
	{
		m_time+=days*86400;
		RebuildTS();
	}
	
	
	
	void DateTime::SetNow()
	{
		m_time = time(NULL);
		RebuildTS();
	}	
	
	DateTime DateTime::Now()
	{
		DateTime ret;
		ret.SetNow();
		return ret;
	}
	
	
	
	//overloads
	//---------
	
	DateTime & DateTime::operator=(const DateTime &eq)
	{
		if(this == &eq)return *this;
		m_time=eq.m_time;
		m_format=eq.m_format;
		
		RebuildTS();
		
		return *this;
	}
	
	
	DateTime & DateTime::operator+=(const TimeSpan &add)
	{
		AddSeconds(add.m_seconds);
		AddMinutes(add.m_minutes);
		AddHours(add.m_hours);
		AddDays(add.m_days);
		
		return *this;
	}
	
	DateTime & DateTime::operator-=(const TimeSpan &add)
	{
		AddSeconds(add.m_seconds*-1);
		AddMinutes(add.m_minutes*-1);
		AddHours(add.m_hours*-1);
		AddDays(add.m_days*-1);
		
		return *this;
	}
	
	
	const DateTime DateTime::operator+(const TimeSpan &add) const
	{
		DateTime res = *this;
		res+=add;
		return res;
	}	
	
	const DateTime DateTime::operator-(const TimeSpan &add) const
	{
		DateTime res = *this;
		res-=add;
		return res;
	}	
	
	
	bool DateTime::operator==(const DateTime &other) const
	{
		return m_time==other.m_time;
	}
	
	bool DateTime::operator!=(const DateTime &other) const
	{
		return m_time!=other.m_time;
	}
		
	bool DateTime::operator<=(const DateTime &other) const
	{
		return (m_time<=other.m_time);
	}
	
	bool DateTime::operator<(const DateTime &other) const
	{
		return (m_time<other.m_time);
	}
	
	bool DateTime::operator>=(const DateTime &other) const
	{		
		return (m_time>=other.m_time);
	}
	
	bool DateTime::operator>(const DateTime &other) const
	{
		return (m_time>other.m_time);
	}
	
	
	//statics:
	//---------
	
	
	
	//formatters:
	
	string DateTime::GDF_d(int val)
	{
		string ret;
		
		if(val<10)ret+="0";

		return ret + BitStream::Int2Str(val);		
	}
	
	string DateTime::GDF_D(int val)
	{
		string ret;
		
		switch(val)
		{
			case 0: ret="Sun"; break;
			case 1: ret="Mon"; break;
			case 2: ret="Tue"; break;
			case 3: ret="Wed"; break;
			case 4: ret="Thu"; break;
			case 5: ret="Fri"; break;
			case 6: ret="Sat"; break;		
		}
		
		return ret;		
	}	


	string DateTime::GDF_j(int val)
	{
		return BitStream::Int2Str(val);
	}

	string DateTime::GDF_l(int val)
	{
		string ret;
		
		switch(val)
		{
			case 0: ret="Sunday"; break;
			case 1: ret="Monday"; break;
			case 2: ret="Tuesday"; break;
			case 3: ret="Wednesday"; break;
			case 4: ret="Thursday"; break;
			case 5: ret="Friday"; break;
			case 6: ret="Saturday"; break;		
		}
		
		return ret;	
	}

	string DateTime::GDF_N(int val)
	{
		return BitStream::Int2Str(val+1);
	}

	string DateTime::GDF_S(int val)
	{
		string ret;
		
		if(val==1)ret="st";
		else if(val==2)ret="nd";
		else if(val==3)ret="rd";
		else if(val>=4 && val<=20)ret="th";
		else if(val==21)ret="st";
		else if(val==22)ret="nd";
		else if(val==23)ret="rd";
		else if(val>=24 && val<=30)ret="th";
		else if(val==31)ret="st";
		
		return ret;	
	}

	string DateTime::GDF_w(int val)
	{
		return BitStream::Int2Str(val);
	}

	string DateTime::GDF_z(int val)
	{
		return BitStream::Int2Str(val);
	}

	string DateTime::GDF_F(int val)
	{
		string ret;
		
		switch(val)
		{
			case 0: ret="January"; break;
			case 1: ret="February"; break;
			case 2: ret="March"; break;
			case 3: ret="April"; break;
			case 4: ret="May"; break;
			case 5: ret="June"; break;
			case 6: ret="July"; break;		
			case 7: ret="August"; break;		
			case 8: ret="September"; break;		
			case 9: ret="October"; break;		
			case 10: ret="November"; break;		
			case 11: ret="December"; break;
		}
		
		return ret;	
	}

	string DateTime::GDF_m(int val)
	{
		string ret;
		
		if(val<10)ret+="0";		
		
		return ret + BitStream::Int2Str(val+1);
	}

	string DateTime::GDF_M(int val)
	{
		string ret;
		
		switch(val)
		{
			case 0: ret="Jan"; break;
			case 1: ret="Feb"; break;
			case 2: ret="Mar"; break;
			case 3: ret="Apr"; break;
			case 4: ret="May"; break;
			case 5: ret="Jun"; break;
			case 6: ret="Jul"; break;		
			case 7: ret="Aug"; break;		
			case 8: ret="Sep"; break;		
			case 9: ret="Oct"; break;		
			case 10: ret="Nov"; break;		
			case 11: ret="Dec"; break;
		}
		
		return ret;	
	}

	string DateTime::GDF_n(int val)
	{
		return BitStream::Int2Str(val+1);
	}

	string DateTime::GDF_t(int year,int month)
	{
		string ret;
		
		switch(month)
		{
			case 0: ret="31"; break;
			case 1: ret=(GDF_L(year)=="1" ? "29" : "28"); break;
			case 2: ret="31"; break;
			case 3: ret="30"; break;
			case 4: ret="31"; break;
			case 5: ret="30"; break;
			case 6: ret="31"; break;		
			case 7: ret="31"; break;		
			case 8: ret="30"; break;		
			case 9: ret="31"; break;		
			case 10: ret="30"; break;		
			case 11: ret="31"; break;
		}
		
		return ret;	
	}

	string DateTime::GDF_L(int val)
	{
		int year = val+1900;
		string ret="0";
		
		// First check if the year is a multiple of 100 
		// If yes, then 
		if(year % 100 == 0)
		{
			if(year % 400 == 0)  // means its a leap year
				ret="1";
		}
		else{
			if(year % 4 == 0)	// means its a leap year 
				ret="1";
		}	
		
		return ret;
	}

	string DateTime::GDF_Y(int val)
	{
		return BitStream::Int2Str(val+1900);
	}

	string DateTime::GDF_y(int val)
	{
		int year=val+1900;

		if(year<2000)year+=100;
		
		year-=2000;		
				
		return (year<10 ? "0" : "") + BitStream::Int2Str(year);
	}

	string DateTime::GDF_a(int val)
	{
		if(val<=12)return "am";
				
		return "pm";
	}
	

	string DateTime::GDF_A(int val)
	{
		if(val<12)return "AM";
				
		return "PM";
	}

	string DateTime::GDF_g(int val)
	{
		if(val>12)val-=12;
		
		if(val==0)val=12;
		
		return BitStream::Int2Str(val);
	}

	string DateTime::GDF_G(int val)
	{
		return BitStream::Int2Str(val);
	}

	string DateTime::GDF_h(int val)
	{
		string ret=GDF_g(val);
		
		if(ret.size()<2)return "0" + ret;
		
		return ret;
	}

	string DateTime::GDF_H(int val)
	{
		string ret=GDF_G(val);
		
		if(ret.size()<2)return "0" + ret;
		
		return ret;
	}

	string DateTime::GDF_i(int val)
	{
		return (val<10 ? "0" : "") + BitStream::Int2Str(val);
	}

	string DateTime::GDF_s(int val)
	{
		return (val<10 ? "0" : "") + BitStream::Int2Str(val);
	}


	string DateTime::GDF_I(int val)
	{
		return BitStream::Int2Str(val);
	}


	string DateTime::GDF_U(long int val)
	{
		return BitStream::Int2Str(val);
	}






	//timespan
	//-----------------------------------------------------------
	
	
	




	TimeSpan::TimeSpan()
	{
		m_seconds=0;
		m_minutes=0;
		m_hours=0;
		m_days=0;
	}
	
	
	void TimeSpan::Defrag()
	{
		while(m_seconds>=60)
		{
			m_minutes++;
			m_seconds-=60;
		}			
		
		while(m_minutes>=60)
		{
			m_hours++;
			m_minutes-=60;
		}		
		
		while(m_hours>=24)
		{
			m_days++;
			m_hours-=24;
		}
	}

	
	
	
	void TimeSpan::SetSeconds(long int seconds)
	{		
		m_seconds=seconds;	
		Defrag();
	}
	
	void TimeSpan::SetMinutes(long double minutes)
	{
		long int newmin=(long int)(floor(minutes));
		m_minutes=newmin;
		
		minutes-=newmin;
				
		Defrag();
		SetSeconds(m_seconds + (long int)(minutes*60));		
	}
	
	void TimeSpan::SetHours(long double hours)
	{
		long int newhour=(long int)(floor(hours));
		m_hours=newhour;
		
		hours-=newhour;		
		
		Defrag();
		SetMinutes(m_minutes + (hours*60));
	}
	
	void TimeSpan::SetDays(long double days)
	{
		long int newday=(long int)(floor(days));
		m_days=newday;
				
		days-=newday;		
		
		Defrag();
		SetHours(m_hours + (days*24));
	}
	
	
	void TimeSpan::AddSeconds(long int seconds)
	{
		if(seconds==0)return;
		long int res = m_seconds+seconds;		
				
		if(res<0)
		{
			AddMinutes(((long double)seconds)/60);		
		}	
		else SetSeconds(res);
	}
	
	void TimeSpan::AddMinutes(long double minutes)
	{
		if(minutes==0)return;
		long double res = m_minutes+minutes;
		
		if(res<0)
		{
			AddHours(minutes/60);
		}
		else SetMinutes(res);
	}
	
	void TimeSpan::AddHours(long double hours)
	{
		
		if(hours==0)return;		
		long double res = m_hours+hours;
		
		if(res<0)
		{
			AddDays(hours/24);
		}
		else SetHours(res);
	}
	
	void TimeSpan::AddDays(long double days)
	{
		if(days==0)return;		
		long double res = m_days+days;
		

		if(res<0)
		{
			m_seconds=0;
			m_minutes=0;
			m_hours=0;
			m_days=0;
		}
		else SetDays(res);
	}	
		


	string TimeSpan::ToStr()
	{
		string ret="";
		
		
		if(m_days>0)
		{
			ret+=(BitStream::Int2Str(m_days) + ".");
		}
		
		if(m_hours<=9)ret+="0";
		ret+=BitStream::Int2Str(m_hours)+":";

		if(m_minutes<=9)ret+="0";
		ret+=BitStream::Int2Str(m_minutes)+":";

		if(m_seconds<=9)ret+="0";
		ret+=BitStream::Int2Str(m_seconds);

		
				
		return ret;
	}
	
	
	//overloads:
	
	TimeSpan & TimeSpan::operator=(const TimeSpan &eq)
	{
		if(this == &eq)return *this;
		m_seconds=eq.m_seconds;
		m_minutes=eq.m_minutes;
		m_hours=eq.m_hours;
		m_days=eq.m_days;
		return *this;
	}
	
	
	TimeSpan & TimeSpan::operator+=(const TimeSpan &add)
	{		
		AddSeconds(add.m_seconds);
		AddMinutes(add.m_minutes);
		AddHours(add.m_hours);
		AddDays(add.m_days);
		
		return *this;
	}
	
	TimeSpan & TimeSpan::operator-=(const TimeSpan &add)
	{
		AddSeconds(add.m_seconds*-1);
		AddMinutes(add.m_minutes*-1);
		AddHours(add.m_hours*-1);
		AddDays(add.m_days*-1);
		
		return *this;
	}
	
	
	const TimeSpan TimeSpan::operator+(const TimeSpan &add) const
	{
		TimeSpan res = *this;
		res+=add;
		return res;
	}	
	
	const TimeSpan TimeSpan::operator-(const TimeSpan &add) const
	{
		TimeSpan res = *this;
		res-=add;
		return res;
	}	
	
	bool TimeSpan::operator==(const TimeSpan &other) const
	{
		return m_days==other.m_days && m_hours==other.m_hours && m_minutes==other.m_minutes && m_seconds == other.m_seconds;
	}
	
	bool TimeSpan::operator!=(const TimeSpan &other) const
	{
		return !(*this==other);
	}
	
	
	bool TimeSpan::operator>=(const TimeSpan &other) const
	{
		if(*this==other)return true;
		return (*this>other);
	}

	
	bool TimeSpan::operator>(const TimeSpan &other) const
	{
		if(m_days>other.m_days)return true;
		else if(m_days<other.m_days)return false;
		
		//at this point days are equal
		
		if(m_hours>other.m_hours)return true;
		else if(m_hours<other.m_hours)return false;
		
		//at this point hours are equal

		if(m_minutes>other.m_minutes)return true;
		else if(m_minutes<other.m_minutes)return false;
		
		//at this point hours are equal
		
		if(m_seconds>other.m_seconds)return true;
		else if(m_seconds<other.m_seconds)return false;	
		
		//at this point seconds are equal, we return false since we want > and not =>

		return false;	
	}
	
	bool TimeSpan::operator<=(const TimeSpan &other) const
	{
		if(*this==other)return true;
		return !(*this>other);
	}
	
	bool TimeSpan::operator<(const TimeSpan &other) const
	{
		return !(*this>=other);
	}
	
	
	//static:
	TimeSpan TimeSpan::FromSeconds(long int seconds)
	{
		TimeSpan ret;
		ret.SetSeconds(seconds);
		return ret;	
	}
	
	TimeSpan TimeSpan::FromMinutes(long double minutes)
	{
		TimeSpan ret;
		ret.SetMinutes(minutes);
		return ret;		
	}
	
	TimeSpan TimeSpan::FromHours(long double hours)
	{
		TimeSpan ret;
		ret.SetHours(hours);
		return ret;		
	}
	
	TimeSpan TimeSpan::FromDays(long double days)
	{
		TimeSpan ret;
		ret.SetDays(days);
		return ret;		
	}
	
	
	TimeSpan TimeSpan::Zero()
	{
		TimeSpan ret;
		ret.m_seconds=0;
		ret.m_minutes=0;
		ret.m_hours=0;
		ret.m_days=0;	
		return ret;
	}
		

	

	
	

	StopWatch::StopWatch()
	{
		t1=0;
		t2=0;
		started=false;
	}

	void StopWatch::Start()
	{
		if(started)return;
		
		
		#ifdef WIN32
			t1 = t1 + time(NULL);
		#else		
			gettimeofday(&tim, NULL);		
			t1=t1+tim.tv_sec+(tim.tv_usec/1000000.0);		
		#endif
		
		
		started=true;
	}
	
	
	
	void StopWatch::Stop()
	{
		if(!started)return;
		
		
		
		#ifdef WIN32
			t2 = t2 + time(NULL);			
		#else
			gettimeofday(&tim, NULL);
			t2=t2+tim.tv_sec+(tim.tv_usec/1000000.0);
		#endif
		
		
		started=false;
	}
	
	
	
	unsigned long int StopWatch::MiSec()
	{
		
		if(started)
		{
			Stop();
			
			unsigned long int ret=0;
			
			#ifdef WIN32
				ret = (unsigned long int)((t2-t1)*1000000);
			#else			
				ret = (unsigned long int)((t2-t1)*1000000);				
			#endif
			
			
			Start();
			
			return ret;
		}
		
		
		return (unsigned long int)((t2-t1)*1000000);;
	}
	
	
	
	unsigned long int StopWatch::MSec()
	{
		return MiSec()/1000;	
	}
	
	unsigned long int StopWatch::Sec()
	{
		return MSec()/1000;	
	}	
		
		
	void StopWatch::Reset()
	{		
		started=false;
		t1=0;
		t2=0;
	}



}

























