namespace rslib
{




MysqlCon::MysqlCon(string host,string user,string pass)
{
		m_host=host;
		m_user=user;
		m_pass=pass;
		m_error="";	
		CON=NULL;
		m_connectionid_str="NULL";
		debug=false;
}


void MysqlCon::SetName(string name)
{
	m_connectionid_str=name;
}

uli  MysqlCon::m_connectionid=0;

void MysqlCon::Debug(int level,string str)
{
	if(debug)cout<<"MySQLcon"<<"["<<m_connectionid_str<<"]: "<< str<<endl;
}


bool MysqlCon::Connect(string host,string user,string pass)
{
	if(CON!=NULL)
	{
		delete CON;
		CON=NULL;
	}

	if(m_connectionid_str=="NULL")
	{
		m_connectionid++;
		m_connectionid_str=BitStream::Int2Str(m_connectionid);	
	}
	
	CON = new mysqlpp::Connection(false);
	
	bool ret=CON->connect(0,host.c_str(),user.c_str(),pass.c_str());	
	if(!ret)
	{
		string errstr=CON->error();
		m_error=errstr;
		Debug(1,"Failed to connect to DB (srv: " + host + " usr: " + user + " ) - " + errstr);
	}
	else Debug(2,"Successfully connected to DB (srv: [" + host + "] usr: [" + user +"])");
		
	return ret;	
}

bool MysqlCon::Connect()
{
	return Connect(m_host,m_user,m_pass);
}

bool MysqlCon::DoQuery(string querystr)
{	
	if(CON==NULL)
	{
		Debug(1,"Unable to perform query: " + querystr + " - Not connected to db!");
		return false;
	}	
	
	mysqlpp::Query thequery = CON->query(querystr);	
	
	m_res = thequery.use();	
	
	thequery.reset();
	
	Debug(4,"Performing SQL query: " + querystr);
	
	if(CON->errnum())
	{
		string err=CON->error();
		m_error=err;
		Debug(1,"ERROR: " + err);
		return false;
	}
	else m_MySqlAffectedRows=thequery.affected_rows();
	
	thequery.reset();
	
	return true;
}

mysqlpp::UseQueryResult MysqlCon::GetResult()
{
	return m_res;
}


bool MysqlCon::CreateDB(string dbname)
{
	if(CON==NULL)
	{
		Debug(1,"Error creating DB: " + dbname + ", CON is NULL");
		return false;
	}
	
	bool ret= CON->create_db(dbname);
	if(!ret)
	{
		string errstr=CON->error();
		m_error=errstr;
		Debug(1,"Failed to create DB " + dbname + ": " + errstr);		
	}
	else Debug(2,"created db " + dbname);
	
	return ret;
}


bool MysqlCon::SelectDB(string dbname)
{
	if(CON==NULL)
	{
		Debug(1,"Error selecting DB: " + dbname + ", CON is NULL");
		return false;
	}
	
	bool ret= CON->select_db(dbname);	
	if(!ret)
	{
		string errstr=CON->error();
		m_error=errstr;
		Debug(1,"Failed to select DB " + dbname + ": " + errstr);
	}
	else Debug(2,"Selecting DB " + dbname);	
	return ret;
}



uli MysqlCon::AffectedRows()
{
	return m_MySqlAffectedRows;
}


void MysqlCon::Disconnect()
{
	//TODO: properly disconnect
	
	CON->disconnect();
	
	delete CON;
	CON=NULL;
}

string MysqlCon::GetError(bool clear=true)
{
	string ret =m_error;
	if(clear)m_error="";
	return ret;
}


bool MysqlCon::IsConnected()
{
	return CON!=NULL; //TODO: check actual connection - this is kinda unsafe
}


}


