/*
======================================================
rslib component: networking implimentors
======================================================
*/


#include "bitstream.h"




namespace rslib
{

    SocketClient::SocketClient(string Pip, int Pport)
    {
        ip=Pip;
        port=Pport;   
        status=0;
		rxbuffer=NULL;
		SetBufferSize(10240);
		m_blockmode=false; 
		
		m_contimeoutsec=5;
		m_contimeoutusec=0;
    }
	
	SocketClient::~SocketClient()
	{
		Disconnect();
		
		if(rxbuffer!=NULL)delete[] rxbuffer;
	}
	
	
	void SocketClient::SetBlockingMode(bool value)
	{
		m_blockmode=value;
	}
	
	void SocketClient::SetConnectTimeout(int secs,int usecs=0)
	{
		m_contimeoutsec=secs;
		m_contimeoutusec=usecs;
	}
	
    
	void SocketClient::SetBufferSize(int psize)
	{
		if(psize<1)psize=1;
		rxbuffersize=psize;
		
		if(rxbuffer!=NULL)delete[] rxbuffer;
		
		try
		{
			rxbuffer = new char[psize];
			
		}
		catch(const std::exception& e)
		{
			cerr<<"Error executing ClientInfo::SetRxBufferSize:"<<e.what()<<endl;
			rxbuffer = new char[2];
			rxbuffersize = 2;
		}
	}
    
    bool SocketClient::IsConnected()
    {
        return status==1;
    }
        
    
    int SocketClient::ShowStatus(bool onlyreturn=true)
    {
        if(!onlyreturn)
		{		
			switch(status)
	        {
	            case 0: cout<<"\nNot connected\n";break;
	            case 1: cout<<"\nConnected\n";break;
	            case 2: cout<<"\nConnect Error\n";break;
	        }
		}
		return status;
    }
	
	
	string SocketClient::GetError(bool clear=true)
	{
		string ret=m_lasterror;
	
		if(clear)m_lasterror="";
		return ret;
	}
    
    
    
    bool SocketClient::Connect()
    {
    	#ifdef WIN32
        WSADATA  wsaData;
        if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) 
		{
            status=2;
			return false;
        }	
    	#endif
    	
    
    
        sockfd = socket(PF_INET, SOCK_STREAM, 0);
    
        if ((he=gethostbyname(ip.c_str())) == NULL)
		{  // get the host info 
    		status=2;
            return false;
        }	
    	
 
		
		#ifndef WIN32
			//declare some temp vars:
			fd_set myset; 
			struct timeval tv; 
			socklen_t lon; 		
			
			
			// Set non-blocking:					
			int valopt;
			long arg = fcntl(sockfd, F_GETFL, NULL); 
			arg |= O_NONBLOCK; 
			fcntl(sockfd, F_SETFL, arg); 
		#endif
		
		// Try to connect:
		
		dest_addr.sin_family = AF_INET;          // host byte order
		dest_addr.sin_port = htons(port);   // short, network byte order
		dest_addr.sin_addr = *((struct in_addr *)he->h_addr);
		memset(dest_addr.sin_zero, '\0', sizeof dest_addr.sin_zero);
		int res = connect(sockfd, (struct sockaddr *)&dest_addr, sizeof(dest_addr)); 

		#ifndef WIN32
			if (res < 0) 
			{ 
			 if (errno == EINPROGRESS) 
			 { 
				tv.tv_sec = m_contimeoutsec;
				tv.tv_usec = m_contimeoutusec; 
				FD_ZERO(&myset); 
				FD_SET(sockfd, &myset); 
				if (select(sockfd+1, NULL, &myset, NULL, &tv) > 0) 
				{ 
				   lon = sizeof(int); 
				   getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon); 
				   if (valopt) 
				   { 
					  status=2;
					  return false;
				   } 
				} 
				else 
				{ 
					  status=2;
					  return false;
				} 
			 } 
			 else 
			 { 
					  status=2;
					  return false;
			 } 
			}
			
			
			// Set to blocking mode again:
			arg = fcntl(sockfd, F_GETFL, NULL); 
			arg &= (~O_NONBLOCK); 
			fcntl(sockfd, F_SETFL, arg); 
			
		#else
			if(res==-1) //TODO: Find a way to setup non blocking sockets in win32 so it does not try to connect forever
			{
				status=2;
				return false;
			}
		#endif
		
    
    	status=1;
    		
        return true;
    }
    
    
    
    bool SocketClient::SendString(string tosend)
    {
        bool willsend=true;
        IsConnected(); //don't need to check return value as it will set status if not connected
        if(status!=1)willsend=false;
    	
    	
    
        if(!willsend)return false;
        
        int res=0;
        int sentsofar=0;
    	while(tosend.size()>0 && sentsofar<tosend.size() && IsConnected())
    	{
    		res=send(sockfd,tosend.c_str(),tosend.size(),MSG_NOSIGNAL);
    
    		if(res<=0)
    		{
        		status=0;
        		return false;
    		}
    		else sentsofar+=res;
    	}	
    
    return true;
    }
    
    
    bool SocketClient::SendBitStream(BitStream bs)
    {
        return SendString(bs.ToStr());
    }
	
    bool SocketClient::SendByteStream(ByteStream bs)
    {
        return SendString(bs.ToStr());
    }	
    
    
	/*Note: for Recv we don't want to loop this function even though that seems like it would be
	better.  Calling recv many times with small buffer is slower then calling it once with a bigger buffer. 
	This function is strictly for if 1 char is needed.
	*/
	bool SocketClient::RecvChar(char & data,uli timeout=15000)
	{
		if(!IsConnected())
		{
			m_lasterror="GetChar: Not connected";
			return false;
		}
		
		char rxbuffer[1]={0x0};

		int recvres=-1;
		
		for(int i=0;i<timeout;i++)
		{			
			recvres = recv(sockfd,rxbuffer,1,MSG_DONTWAIT);
			if(recvres>0)break;
			Wait(1000);//wait 1ms
		}
		
		
		//Check for error (note: errno is part of raw sockets)
		if(recvres<1)
		{
			if(errno==EAGAIN || errno==EWOULDBLOCK)
			{
				m_lasterror="GetChar: Did not receive char within timeout";
				return false;
			}
			else 
			{
				status=0;
				m_lasterror="GetChar: Socket error No. " + BitStream::Int2Str(errno);
				return false;
			}
		}
		
		data=rxbuffer[0];
		
		return true;
	}
    
	bool SocketClient::Recv(BitStream & data,uli bytes,uli timeout=15000)
	{
		BitStream bs;
		bool ret = Recv(bs,bytes,timeout=15000);
		data.WriteString(bs.ToStr());
		return ret;
	}
    
    bool SocketClient::Recv(ByteStream & data,uli bytes,uli timeout=15000)
    {
		if(!IsConnected())
		{
			m_lasterror="Recv: Not connected";
			return false;
		}
		
        ByteStream buffer; //declare bitstream buffer - will only modify param if we get all data
		
        if(bytes<=0 || status!=1)return false; //if we specified bad arguments, or if we're not connected
		if(bytes>rxbuffersize)bytes=rxbuffersize; //make sure we are not trying to get more then what the buffer can hold
		
	
   
		uli timeoutleft = timeout;
        
		int recvres=1;
		do //loop until we gotten all data.  This is because recv is not guaranteed to actually return all in one shot
		{
			while(timeoutleft>0) //loop until timeout is is 0
			{
				if(timeoutleft>0)timeoutleft--;

				recvres = recv(sockfd,rxbuffer,bytes,MSG_DONTWAIT); //non blocking recv
				
				if(recvres>0)bytes-=recvres;//deduct bytes from counter

				
				if(recvres!=-1)break; //breat out if return value is not erronaus
				Wait(1000);//wait 1ms				
			}
			
			
			if(recvres==0 && bytes>0)Wait(1000); //if bytes are 0 in this case we need to wait for more data
			
			
			//check for error:
			if(recvres==-1 || (recvres==0 && bytes>0 && timeoutleft==0))
			{
				m_lasterror="Recv: Did not receive char within timeout, or the socket may be closed.";
				return false;
			}
			
			
			
			for(int i=0;i<recvres;i++) buffer.WriteInt8(rxbuffer[i]); //write to buffer

			
		}while(bytes>0);
	
		data.WriteByteStream(buffer);
    
        return true;
    }
    
    
    bool SocketClient::Recv(string & data,uli bytes,uli timeout=15000)
    {
        BitStream bs;
        if(Recv(bs,bytes,timeout))
		{
			data+=bs.ToStr();
			return true;
		}
		
        return false;
    }

	bool SocketClient::RecvUntilChar(BitStream & data, char stopchar,uli timeout=15000,uli max=1048576)
	{
		ByteStream bs;
		bool ret = RecvUntilChar(bs, stopchar,timeout,max);
		data.WriteString(bs.ToStr());
		return ret;
	}	
	
    bool SocketClient::RecvUntilChar(ByteStream & data, char stopchar,uli timeout=15000,uli max=1048576)
	{
		ByteStream ret;
	
		char tmpchar=0x0;
		for(int i=max;i>0;i--)
		{
			if(!RecvChar(tmpchar,timeout))
			{
				return false;
			}
			if(tmpchar==stopchar)
			{
				data.WriteByteStream(ret);
				return true;
			}
			ret.WriteInt8(tmpchar);
		}
		
		m_lasterror="RecvUntilChar: Reached max before finding requested char.  Warning: some data may still be left in buffer.";
		
		return false;
	}	
	
	
	bool SocketClient::RecvUntilChar(string & data, char stopchar,uli timeout=15000,uli max=1048576)
	{
		BitStream ret;
		if(!RecvUntilChar(ret,stopchar,timeout,max))
		{
			return false;
		}
		
		data = ret.ToStr();
		
		return true;	
	}

    
    void SocketClient::Disconnect()
    {
        status=0;
        #ifdef linux
            close(sockfd);
        #endif
        #ifdef WIN32
            closesocket(sockfd);
        #endif
    }
    
    

    
    
    
    
    
    //=========================================================================================    
    //================================== SERVER STUFF =========================================
	//=========================================================================================

	//ServerClient class:
	
	ServerClient::ServerClient(ClientInfo * ci)
	{
		m_ci = ci;
		m_null=false;
		RefCount(1);
	}
	

	ServerClient::ServerClient()
	{
		m_null=true;
		m_ci=NULL;
	}
	
	ServerClient::~ServerClient()
	{
		if(RefCount(-1)==0)Delete(); //we are done with the ClientInfo object, signal SocketServer about it.
	}
	
	int ServerClient::RefCount(int c)
	{
		if(m_ci!=NULL)
		{			
			return m_ci->RefCount(c);
		}
		else Delete(); //this should never happen...
		return 0;
	}
			
	bool ServerClient::IsConnected()
	{		
		if(m_null)return false;
		if(m_ci==NULL)return false;
		
		return m_ci->IsConnected();
	}
	
	
	void ServerClient::Disconnect()
	{
		if(m_null)return;
		
		m_ci->Disconnect();
	}
	
	
	void ServerClient::Delete()
	{
		if(m_null)return;
		
		m_null=true;
		m_ci->deleted=true;	
	}
	
	
	bool ServerClient::Lock(bool block=true)
	{		
		if(!IsConnected())return false;
		
		#ifdef rslib_pthread	
			m_ci->Lock(block);			
		#endif

		return true;
	}
	
	void ServerClient::Unlock()
	{
		if(!IsConnected())return;
		
		#ifdef rslib_pthread
		m_ci->Unlock();
		#endif
	}
	
    unsigned long int ServerClient::QueueSize()
    {
        if(m_null)return 0;
		return (m_ci->dataqueue.Size());
    }

    unsigned long int ServerClient::QueueSizeBits()
    {
        if(m_null)return 0;
		return (m_ci->dataqueue.Size());
    }	
	
    string ServerClient::GetQueue()
    {
        if(m_null)return "";
		
		return m_ci->dataqueue.ToStr();
    }
    
    bool ServerClient::PopData(string & data,int bytes)
    {
        if(!IsConnected())return false;
		
		Lock();
		
		bool ret=true;
		
		if(bytes>QueueSize())ret=false;
        
		if(ret)
		{			
			data+=m_ci->dataqueue.PopString_s(bytes);
		}
		
		Unlock();		

		return ret;
    }
	
    bool ServerClient::PopBitStream(BitStream & data, int bytes)
	{
		if(!IsConnected())return false;
		string tmp;
		bool ret = PopData(tmp,bytes);
		
		if(ret)data.WriteString(tmp);
		
		return ret;
	}
	
	
    bool ServerClient::PopByteStream(ByteStream & data, int bytes)
	{
		if(!IsConnected())return false;
		string tmp;
		bool ret = PopData(tmp,bytes);
		
		if(ret)data.WriteString(tmp);
		
		return ret;
	}	
    
	
	void ServerClient::TXdata(string bytes)
    {
        if(!IsConnected() || bytes.size()<1)return;
		
		Lock();
		
        int tosend=bytes.size();
		
		//TODO: ensure the whole string gets sent
        
		int ret = send(m_ci->FD,bytes.c_str(),bytes.size(),MSG_NOSIGNAL);
		
		
		if(ret<1)
		{
			m_ci->connected=false;
		}
		
		Unlock();
    }
	
	
    bool ServerClient::RXdata()
    {
        if(!IsConnected())return false;
		
        return m_ci->RXdata();
    }	
	
    
	string ServerClient::GetIPstr(bool includeport=false)
    {
        if(m_null)return "";
		      
        return m_ci->GetIPstr(includeport);
    }
	
	BitStream ServerClient::GetIP()
	{
		if(m_null)return BitStream();
		return m_ci->GetIP();
	}
	
	int ServerClient::GetPort()
	{		
		 if(m_null)return 0;
		return m_ci->GetPort();
	}
	
	string ServerClient::GetPortstr()
	{		
		 if(m_null)return "";
		return m_ci->GetPortstr(); 
	}
	
	int ServerClient::GetFD()
	{
		if(m_null)return -1;
		
		return m_ci->FD;
	}
	
	
    ServerClient& ServerClient::operator=(const ServerClient& tw)
    {
    	if (this == &tw || tw.m_null) return *this;    //make sure we're not assigning ourselves to ourselves
		
		m_null=false;
		m_ci = tw.m_ci;
		
		RefCount(1);
		
    	return *this;
    }
	
	ServerClient::ServerClient(const ServerClient & sc)
	{
		*this=sc;	
	}
	
	
	bool ServerClient::operator==(const ServerClient & comp)
	{
		return(m_ci==comp.m_ci);
	}
	
	
    //ClientInfo class:
                
    ClientInfo::ClientInfo(SocketServer * pserver,int pFD,struct sockaddr_in paddr)
    {
        server=pserver;
		FD=pFD;
		m_RefCount=0;
		
		//Set IP:
        BitStream tmp;
        tmp.WriteInt32((unsigned long int)paddr.sin_addr.s_addr);
		
		//this puts it backwards so we need to reorder it:		
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		
		//Set port:  (we don't have to clear tmp as the pops cleared it alrady)
		tmp.WriteInt16(paddr.sin_port);
		
		//port is in reverse so we'll just pop it backwards to new bitstream then convert to int
		BitStream tmp2;
		
		tmp2.PushInt8_e(tmp.PopInt8_e());
		tmp2.PushInt8_e(tmp.PopInt8_e());
		Port=tmp2.PopInt16_e();
		
		//set IP in STR format:
		
		IpAddressSTR="";
		IpAddressSTR+=BitStream::Int2Str(IpAddress.ReadInt8(0))+".";
		IpAddressSTR+=BitStream::Int2Str(IpAddress.ReadInt8(8))+".";
		IpAddressSTR+=BitStream::Int2Str(IpAddress.ReadInt8(16))+".";
		IpAddressSTR+=BitStream::Int2Str(IpAddress.ReadInt8(24));		
				
        connected=true;	
		deleted=false;
		disconnected=false;
		rxtmp=NULL;
		SetRxBufferSize(pserver->defaultbuffersize);
    }
	
	 ClientInfo::~ClientInfo()
	 {
		if(rxtmp!=NULL)delete[] rxtmp;
	 }
	 
	 
	 int ClientInfo::RefCount(int inc)
	 {
		if(inc==1)m_RefCount++;
		else if(inc==-1)m_RefCount--;
		
		return m_RefCount;	 
	 }
	 
	 
	 void ClientInfo::SetRxBufferSize(int bytes)
	 {
		if(rxtmp!=NULL)delete[] rxtmp;
		
		try
		{
			rxtmp = new char[bytes];
			rxtmpsize = bytes;	
			
		}
		catch(const std::exception& e)
		{
			cerr<<"Error executing ClientInfo::SetRxBufferSize:"<<e.what()<<endl;
			rxtmp = new char[2];
			rxtmpsize = 2;
		}			
	 }
	
	
	bool ClientInfo::Lock(bool block=true)
	{		
		#ifdef rslib_pthread	
			if(block)mutex.Lock();
			else return mutex.TryLock();
		#endif

		return true;
	}

	
	void ClientInfo::Unlock()
	{
		#ifdef rslib_pthread
		mutex.UnLock();
		#endif
	}
	
    
    bool ClientInfo::IsConnected()
    {        
        return connected;
    }
	
	void ClientInfo::Disconnect()
	{
		if(!connected)return;
		
		connected=false; //Just mark as disconnected - DO NOT set disconnect as true, or it will never actually disconnect
	}
    
    unsigned long int ClientInfo::QueueSize()
    {
        return (dataqueue.Size());
    }
    
    bool ClientInfo::RXdata()
    {
		if(!IsConnected())return false; 	
			
		rxret = recv(FD,rxtmp,rxtmpsize,MSG_DONTWAIT);
	
		if(rxret==-1)
		{
			#ifdef WIN32
			errno = WSAGetLastError();
			#endif
			
			if(errno==EAGAIN || errno==EWOULDBLOCK) return false;//no data
			else rxret=0; //set to disconnect status
		}
		
		if(rxret==0) //disconnected client
		{
			connected=false;
			return false;
		}
								
		dataqueue.Lock();
		for(int i=0;i<rxret;i++)
		{
			dataqueue.PushInt8_e((unsigned int)(unsigned char)rxtmp[i]);
		}
		dataqueue.Unlock();
		
		
		return true;
    }

    
    string ClientInfo::GetIPstr(bool includeport=false)
    {
        string ret=IpAddressSTR;
        
        if(includeport)
        {
			ret+=":"+GetPortstr(); 				
        }
        		
        return ret;
    }
	
	BitStream ClientInfo::GetIP()
	{
		return IpAddress;
	}
	
	int ClientInfo::GetPort()
	{		
		return Port;
	}
	
	string ClientInfo::GetPortstr()
	{		
		return BitStream::Int2Str(GetPort());  
	}
	




	
	
	

    
    //socketserver class:

    
    SocketServer::SocketServer()
    {
        BackLog=10;   
		listening=false;
		debugmode=false;	
		defaultbuffersize=10240;
		TrackClientDC=false;
    }
    
    
    SocketServer::~SocketServer()
    {
        Shutdown(true);
    }
	
	
	bool SocketServer::Lock(bool block=true)
	{
		
		#ifdef rslib_pthread		
		if(block)mutex.Lock();
		else return mutex.TryLock();
		#endif

		return true;
	}

	
	void SocketServer::Unlock()
	{
		#ifdef rslib_pthread
		mutex.UnLock();
		#endif
	}	
    
    void SocketServer::SetBacklog(int bl)
    {
        if(bl<1)bl=1;
        BackLog=bl;
    }
	
	void SocketServer::SetBufferSize (int psize)
	{
		if(psize<1)psize=1;
		defaultbuffersize=psize;
	}	
    
    string SocketServer::GetMessage()
    {
        string msg=Message;
        Message="";
        return msg;
    }
    
    int SocketServer::GetConnectionNum()
    {
        return Clients.GetSize();
    }
    
    void SocketServer::SetMessage(string msg,bool error)
    {
        if(msg=="")Message="";
        else Message = (error ? "ERROR: " : "INFO: ") + msg;

		if(debugmode && Message!="")cout<<Message<<endl;
    }
    
    void SocketServer::ClientAdd(ClientInfo * client)
    {
        NewClients.Add(client);
    }
	
    void SocketServer::ClientDisconnected(ClientInfo * client)	
	{
        if(client==NULL || client->disconnected)return;
		if(TrackClientDC)DCedClients.Add(client->GetIPstr(true));
		SetMessage(client->GetIPstr(true) + " Disconnected",false);

		client->disconnected=true;		
		
		#ifdef linux
            close(client->FD);
        #endif
        #ifdef WIN32
            closesocket(client->FD);
        #endif

        return;
	}
	
	void SocketServer::ClientDelete(ClientInfo * client)
	{
		
		if(Clients.Erase(client))
		{
			delete client;
			return;
		}
		
		if(NewClients.Erase(client))
		{
			delete client;
			return;
		}		
	}

	
    
    
        
    bool SocketServer::Listen(int port,string ip="")
    {
        if(listening)
		{
			SetMessage("Already listening",true);
			return false;
		}		
		
		listening=true;
		
		#ifdef WIN32
		
		WSADATA info;
		if (WSAStartup(MAKEWORD(2,0), &info))		
		{
			SetMessage("WSAStartup failed",true);
			return false;
		}

		#endif
		
        if((sockfd = socket(PF_INET, SOCK_STREAM, 0))==-1)
        {
            SetMessage("Failed to create socket",true);
            return false;
        }
		
		
		
		
		#ifdef WIN32
			//enable non blocking sockets for win32:
			u_long arg = 1;
			ioctlsocket(sockfd, FIONBIO, &arg);	
			
	        //also this part needs to be different in win32
			const char unknown=1;
			
	        if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &unknown, sizeof(int)) == -1)
	        {
	            SetMessage("Failed to set socket options",true);
	            return false;
	        }
		#elif linux
			
	        
	        const int unknown=1;
			
	        if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &unknown, sizeof(int)) == -1)
	        {
	            SetMessage("Failed to set socket options",true);
	            return false;
	        }
		
		#endif
    
        my_addr.sin_family = AF_INET;         // host byte order
        my_addr.sin_port = htons(port);     // short, network byte order
        my_addr.sin_addr.s_addr = (ip=="" ? INADDR_ANY : inet_addr(ip.c_str()));
        memset(my_addr.sin_zero, '\0', sizeof my_addr.sin_zero);
        
        if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof my_addr) == -1) 
        {
            SetMessage("Could not bind to the network, port already in use?",true);
            return false;
        }
        
        if (listen(sockfd, BackLog) == -1) 
        {
            SetMessage("Could not listen for data",true);
            return false;
        }
        
        SetMessage("Listening on " + (ip=="" ? "[all]" : ip),false);

        return true;
    }
    
    bool SocketServer::Poll(int polltimeout=1)
    {
        //check if there is a client waiting to connect:	
		
		struct timeval tv;
	    fd_set readfds;

	    tv.tv_sec = 0;
	    tv.tv_usec = polltimeout;

	    FD_ZERO(&readfds);
	    FD_SET(sockfd, &readfds);

	    select(sockfd+1, &readfds, NULL, NULL, &tv);
		
		if(FD_ISSET(sockfd, &readfds))
		{
			//handle new connection
			int tmpFD; 
			struct sockaddr_in tmpaddr;  
			socklen_t sin_size = sizeof tmpaddr;
			
			if ((tmpFD = accept(sockfd, (struct sockaddr *)&tmpaddr,  &sin_size)) == -1) 
			{
				SetMessage("failed at accept()",true);
				return false;
			}
						
			ClientInfo * client = new ClientInfo(this,tmpFD,tmpaddr);
			
			ClientAdd(client);			
			
			SetMessage("Client " + client->GetIPstr(true) + " has connected",false);     
			
			return true;                    
		} 
        
        PollData(true);	
	
        SetMessage("",false);
        return false;
    }
	
	
	
	
    void SocketServer::PollData(bool connectionsonly=false)
    {
		ClientInfo * tmpclient=NULL;		
		
		for(int i=0;i<Clients.GetSize();i++)
		{
			if(!Clients.Get(tmpclient,i))continue;			
			if(tmpclient==NULL)continue;			
			
			if(!tmpclient->Lock(false))
			{
				continue;
			}
			
			//check if the client is diconnected, and call the dc function:
			if(!tmpclient->IsConnected())ClientDisconnected(tmpclient);			
			
			//poll data if we chose to do that too:
			if(!connectionsonly)tmpclient->RXdata();
			
			//client is marked for delete, so we delete it. This frees up the memory
			if(tmpclient->deleted)
			{			
				ClientDelete(tmpclient);
				continue;
			}			
			
			tmpclient->Unlock();			
		}
	}
	
    
    void SocketServer::Shutdown(bool deallocate=false)
    {
		ClientInfo *tmp=NULL;

        for(int i=0;NewClients.Get(tmp,i);i++)
        {
            if(deallocate && tmp!=NULL)
			{
				ClientDisconnected(tmp);
				ClientDelete(tmp);
			}
        }	

        for(int i=0;Clients.Get(tmp,i);i++)
        {
            if(deallocate && tmp!=NULL)
			{
				ClientDisconnected(tmp);
				ClientDelete(tmp);
			}
		}
		
		#ifdef linux
			close(sockfd);
			shutdown(sockfd,SHUT_RDWR);
		#endif
		#ifdef WIN32
			closesocket(sockfd);
		#endif		
    }
    
	
    xVector<ServerClient> SocketServer::GetConnectedClients(bool onlywithdata=false)
    {
        xVector<ServerClient> ret;

		ClientInfo * tmp=NULL;
        for(int i=0;Clients.Get(tmp,i);i++)
        {
            if(tmp->IsConnected())
            {
                if(tmp->QueueSize()>0 && onlywithdata)ret.Add(ServerClient(tmp),-1);
                else ret.Add(ServerClient(tmp),-1);
            }
        }
        return ret;    
    }
    

	bool SocketServer::GetNewClient(ServerClient & client)
	{
		if(NewClients.GetSize()<1)return false;
		
		ClientInfo * tmp = NULL;
		
		if(NewClients.Get(tmp,0))
		{
			Clients.Add(tmp,-1);
			NewClients.EraseAt(0,false);			
			client = ServerClient(tmp);
			return true;
		}		
		
		return false;
	}
	
	
	int SocketServer::GetConNum()
	{
		return NewClients.GetSize()+Clients.GetSize();
	}
	
	bool SocketServer::GetDCedClient(string & client)
	{
		return DCedClients.PopFrom(client,DCedClients.GetSize()-1);
	}
	
	
	void SocketServer::DeleteDeadClients()
	{
		if(NewClients.GetSize()<1)return;
		
		ClientInfo *tmp=NULL;

        for(int i=0;NewClients.Get(tmp,i);i++)
        {
            if(tmp!=NULL && tmp->disconnected)
			{				
				ClientDelete(tmp);
			}
        }	

        for(int i=0;Clients.Get(tmp,i);i++)
        {
            if(tmp!=NULL && tmp->disconnected)
			{
				ClientDelete(tmp);
			}
        }		
	}
	
	
	
	
	
	//Statics:
	
	void SocketServer::SetFDlimit(int limit=65535)
	{
		#ifdef linux
		struct rlimit fslimit;
		fslimit.rlim_cur = limit;
		fslimit.rlim_max = limit;	
		setrlimit(RLIMIT_NOFILE, &fslimit);	
		#endif
	}
	
	//note: this returns true all the time if we are resolving an IP, as it is already resolved.  (ex: does not do reverse)
	bool ResolveHost(string host,BitStream & IpAddress)
	{
		struct hostent *he;  
		struct in_addr addr;
		
		if((he = gethostbyname(host.c_str()))==NULL)
		{
			return false;
		}
		
		addr.s_addr = *(u_long *) he->h_addr_list[0];
		
		IpAddress.Clear();				
        BitStream tmp;	
		
        tmp.WriteInt32((unsigned long int)addr.s_addr);
		
		//this puts it backwards so we need to reorder it:		
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());
		IpAddress.PushInt8_e(tmp.PopInt8_e());

		return true;
	}
	
	
	bool ValidateIP(string ip)
	{
		xVector<string> exploded = Parser::Explode(ip,".");
		
		if(exploded.GetSize()!=4)return false;
		
		string tmp;
		for(int i=0;exploded.Get(tmp,i);i++)
		{
			int octet = BitStream::Str2Int(tmp);
			if(octet<0 || octet>255)return false;
		}
		
		return true;
	}	
	
	
	bool ValidateHostName(string hostname,bool requireresolve=false,bool acceptip=false)
	{
		if(acceptip && ValidateIP(hostname))return true;		
		
		if(!ValidateString(hostname, VALFLG_NOSPACE | VALFLG_FILENAME | VALFLG_NOEMTPY,false) && !ValidateIP(hostname))return false;
		
		if(hostname[0]=='.')return false;
		
		if(requireresolve)
		{
			BitStream ip;
			if(!ResolveHost(hostname,ip))return false;
		}
	
		return true;
	}
	
	
	
	
	

	
	
	
	
	
}
