/*
======================================================
rslib component:  userio implimentors
======================================================
*/



namespace rslib
{

	void Wait(int microseconds)
	{
	   #ifdef WIN32
       Sleep((microseconds/1000)+1); //not the best approach but somewhat close enough (a typical person wont notice the difference, we're talking miliseconds here)
       #else
       usleep(microseconds);
       #endif
	}
	
	void ClearScreen()
	{
        #ifdef WIN32
        system("cls");
        #endif
        #ifdef linux
        system("clear");
        #endif        
    }
	

	void PressReturn(string message)
	{
    	cout<<endl<<message;

		char data;
		do
		{
    		data = GetChar();
		}while(data!='\n' && data!='\r');
        cout<<endl;
	}
	
	unsigned int GetChar()
	{
        #ifdef WIN32
            return getch();
        #endif
        #ifdef linux 
            int ch;
            struct termios oldt, newt;    					
            tcgetattr ( STDIN_FILENO, &oldt );				
            newt = oldt;									
            newt.c_lflag &= ~( ICANON | ECHO );			
			
			newt.c_cc[VINTR]    = 0;     /* Ctrl-c */ 			
			
            tcsetattr ( STDIN_FILENO, TCSANOW, &newt );		
            ch = getchar();			
            tcsetattr ( STDIN_FILENO, TCSANOW, &oldt ); 	
			
			if(ch==3)exit(0); //catch ctrl+c
			
            return ch;
        #endif   
    }
	

	void Pause()
	{
        GetChar();
	}


	
	string PromptPassword(string prompt,string def="", int max=1024)
	{
		return PromptString(prompt,def,max,true);	
	}
	


	string PromptString(string prompt,string def="",int max=1024,bool password=false)
	{             
		if(def!="")prompt+=" [" + (password ? "*******" : def) + "]";
		prompt+=": ";
		cout<<prompt;		
		
		string thestring=def;
		
		if(password)
		{
			char thechar;
			
			while(1)
			{
				thechar=GetChar();
				
				if(thechar==127)
				{					
					if(thestring.size()>0 && def=="")
					{
						thestring = thestring.substr(0,thestring.size()-1);
						cout<<"\b \b";
					}
					else if(def!="")
					{
						cout<<"\b\b\b\b\b\b\b\b\b\b\b\b            \b\b\b\b\b\b\b\b\b\b\b\b: ";
						
						thestring="";
						def="";
					}
				}
				else if(thechar==10)
				{
					cout<<endl;
					break;
				}
				else 
				{					
					if(def!="")
					{
						thestring="";
						def="";
					}
					
					thestring.append(1,thechar);
					cout<<"*";
				}
			}
		}		
		else
		{
			char * tmpcstr = new char[max];
			memset(tmpcstr,0,max);
			cin.getline(tmpcstr,max);			
			thestring = tmpcstr;		
			delete[] tmpcstr;
		}
		
		
		
		if(thestring.size()>max)thestring=thestring.substr(0,max);
			
        cout<<endl;

    	if(thestring=="" || thestring.size()<1)return def;
    
    	return thestring;
	}




	long int PromptInt(string prompt,string def)
	{
    	string intstr = PromptString(prompt,def,25);
    
    	return atoi(intstr.c_str());
	}

	long int PromptInt(string prompt)
	{
	   return PromptInt(prompt,"");
	}



	bool PromptBool(string prompt,string def)
	{
	string res=PromptString(prompt,def,1);

	if(tolower(res[0])=='y' || tolower(res[0])=='t' || tolower(res[0])=='1')return true;
	else return false;
	}
	
	
	
	
	string WrapText(string text,int indent=0,int width=79)
	{
		if(indent<0)indent=0;
		if(width<1)width=1;
		
		xVector<string> exploded = Parser::Explode(text," ");

		string ret;
		string tmp;
		string toprint;
		
		for(int i=0;exploded.Get(tmp,i);i++)
		{
			if((toprint.size()+tmp.size())>(width-indent))
			{
				ret+="\n";
				for(int ii=0;ii<indent;ii++)ret+=" ";//indent
				ret+=toprint;
				toprint="";
			}
			if(tmp!=" ") toprint+= tmp + " ";
		}
		
		ret+="\n";
		for(int ii=0;ii<indent;ii++)ret+=" ";//indent
		ret+=toprint;
		return ret;
	}
	


	//returns a string the size of "size" no matter what is contained in str
	string ForceSize(string str,int size,char fillerchar=' ')
	{
	if(str.size()>size)return str.substr(0,size);

	int charstoadd=size-str.size();

	for(int i=0;i<charstoadd;i++)str.append(1,fillerchar);

	return str;
	}
	
	
	
	bool ValidateString(string & tovalidate,unsigned long int flags,bool changetovalidate=false)
	{
		string change=tovalidate;
		bool valid=true;
		
		
		
		if(flags & VALFLG_NOCAPS)
		{
			for(int i=0;i<change.size();i++)
			{
				if(change[i]>=65 && change[i]<=90)
				{
					valid=false;
					change[i]=change[i]+32;
				}
			}		
		}
		
		
		
		if(flags & VALFLG_NONUM)
		{
			for(int i=0;i<change.size();i++)
			{
				if(change[i]>=48 && change[i]<=57)
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}		
		}
		
		
		
		if(flags & VALFLG_NOSPACE)
		{
			for(int i=0;i<change.size();i++)
			{
				if(change[i]==32 || change[i]==9) //include tab in this as well
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}	
		}
		
		
		
		if(flags & VALFLG_SQL)
		{
			const char SC[] = {'\0','\'','\"','\b','\n','\r','\t','\\','\%','_'};	//special chars that should be escaped
			const int SCsize=10;	//set to size of above
			bool docontinue=false;
			
			for(int i=0;i<change.size();i++)
			{
				docontinue=false;
				
				for(int ii=0;ii<SCsize;ii++)
				{				
					if(change[i]==SC[ii] && SC[ii]!='\\')
					{
						docontinue=true;
						if((i>0 && change[i-1]=='\\') || (i<(change.size()-1) && change[i+1]=='\\'))continue;
						
						valid=false;
						change.insert(i,"\\");
						i++;
					}
				}
				
				if(docontinue)continue;
				
				if(change[i]=='\\')
				{
					for(int ii=0;ii<SCsize;ii++)
					{	
						if(i<(change.size()-1) && change[i+1]==SC[ii])
						{							
							docontinue=true;
							i++;
							break;
						}											
					}
					
					if(docontinue)continue;

					change.insert(i,"\\");
					i++;					
				}							
			}	
		}	
		
		
		
		if(flags & VALFLG_FILENAME)
		{
			for(int i=0;i<change.size();i++)
			{
				if(change[i]==34 || change[i]==39 || change[i]==42 || change[i]==47 || change[i]==58 || change[i]==59 || change[i]==60 || change[i]==62 || change[i]==63 || change[i]==92 || change[i]==124 )
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}
		}
		
		
		if(flags & VALFLG_FILEPATH)
		{
			
			for(int i=0;i<change.size();i++)
			{
				if(change[i]==34 || change[i]==39 || change[i]==42 || change[i]==58 || change[i]==59 || change[i]==60 || change[i]==62 || change[i]==63 || change[i]==124 )
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}
		}		
		
		
		
		if((flags & VALFLG_SPECIALCHARS) || (flags & VALFLG_STRICTUSERNAME))
		{
			for(int i=0;i<change.size();i++)
			{
				if(change[i]<=31 || change[i]>=127)
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}
		}
		
		
		if(flags & VALFLG_STRICTUSERNAME)
		{
			for(int i=0;i<change.size();i++)
			{
				if((change[i]>=33 && change[i]<=44) || change[i]==96 || change[i]==46 || change[i]<=47 || (change[i]>=58 && change[i]<=64) || (change[i]>=91 && change[i]<=94) || (change[i]>=123 && change[i]<=126))
				{
					valid=false;
					change.erase(i,1);
					i--;
				}
			}
		}
				
		
		
		//This should always be last!
		if(flags & VALFLG_NOEMTPY)
		{
			if(change.size()<=0)valid=false;		
		}
		
		
	
		if(changetovalidate)tovalidate=change;
		return valid;
	}
	
	
	bool ValidateNumber(int & tovalidate,long int min,long int max,bool changetovalidate)
	{
		int change=tovalidate;
		bool valid=true;
		
		if(change<min)
		{
			valid=false;
			change=min;
		}
		else if(change>max)
		{
			valid=false;
			change=max;
		}		
		
		if(changetovalidate)tovalidate=change;
	
		return valid;
	}
	
	
	string FixSQL(string tofix)
	{
		ValidateString(tofix,VALFLG_SQL,true);
		
		return tofix;
	}
	


}





