/*****************************************************
EnviroState: Environmental Monitoring System
http://www.iceteks.com

This program is freeware and can be used in any
scenario such as personal, commercial, industrial, 
healthcare, telecommunications, etc. and is provided
"as is" and we shall not be held liable for any reason.

Report any bugs at www.iceteks.com.

******************************************************/


/* USER EDITABLE VARS */

#define LASTAUTHOR "Red Squirrel"
#define VERSION "1.0.0"
#define LASTUPDATE "Oct 12 2012"

/* END USER EDITABLE VARS */



#include <iostream>
#include <fstream>
#include <sstream>

#include <fcntl.h> 
#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>


#define rslib_network
#include "3rdpartyinc/rslib/_allrslib.h"
//Note: Rslib is a custom library that I use throughout stuff I code and it is included in this package to make it easier to deploy this application.


using namespace std;
using namespace rslib;


#include "includes.h"
#include "sources.h"



int main(int argc, char* argv[])
{
	ClArgs args(argc,argv,"");    
  
	if(args.HasArg("help"))
	{
		cout<<endl;
		cout<<"Envirostate "<<VERSION<<" (last updated "<<LASTUPDATE<<" by "<<LASTAUTHOR<<")"<<endl;
		cout<<"Environmental monitoring system."<<endl<<endl;

		cout<<"-help                    This message"<<endl;
		cout<<"-server                  Run as server"<<endl;
		cout<<"-agent                   Run as agent"<<endl;
		cout<<"-service                 Suppress all output"<<endl; 
		cout<<"-confpath=[value]        Alternate configuration path (default: /etc/envirostate)"<<endl;
		cout<<"-genconf                 Generates a basic configuration / samples (not implimented yet)"<<endl;
		cout<<"-debuglevel              Debug Level (0-6) 6 is more verbose"<<endl;
		cout<<"-confdebuglevel          Debug Level (0-5) for config file load/read process"<<endl;

		cout<<endl;
		return 0;
	}

  
	//declare/prep main stuff:
	//------------------------

	Core * CORE=NULL;
	AppLogger * LOGGER=NULL; 
	
	


	

	string pidfilename = "";
	bool runloop=true;
	
	if(args.HasArg("server"))
	{
		LOGGER = new AppLogger("/var/log/envirostate-server");
		LOGGER->CheckRotation(1000000);				
		LOGGER->ConsoleOut=!args.HasArg("service");  
		LOGGER->DebugLevel=args.HasArg("debuglevel") ? args.GetValueInt("debuglevel") : 1;		
		
		pidfilename="envirostate-server.pid";
		CORE = new ServerCore(args,LOGGER);
	}
	else if(args.HasArg("agent"))
	{
		LOGGER = new AppLogger("/var/log/envirostate-agent");
		LOGGER->CheckRotation(1000000);		
		LOGGER->ConsoleOut=!args.HasArg("service");  
		LOGGER->DebugLevel=args.HasArg("debuglevel") ? args.GetValueInt("debuglevel") : 1;		
		
		pidfilename="envirostate-agent.pid";		
		CORE = new AgentCore(args,LOGGER);
	}
	else
	{
		runloop=false;
	}
	
	
	
  
	if(runloop)
	{
		LOGGER->Log(1,"Envirostate " + string(VERSION) + " starting with args: " + args.ShowArgs(" "));
		
		//create pid file:
		fstream pidout(pidfilename.c_str(),ios::out);	
		pidout<<time(NULL);	
		pidout.close();
		
	
	
		//load config and initalize core
		//-------------------------------
		if(!CORE->Load())
		{
			LOGGER->Log(0,"Envirostate unable to load core, exiting.  Check log for details.");
			return 0;
		}


		LOGGER->Log(3,"Entering main loop");
		//start the main loop:
		bool Running=!args.HasArg("runonce");
		do
		{
		}while(CORE->Poll() && Running);
	
	
		unlink(pidfilename.c_str());
		
		LOGGER->Log(1,"Envirostate stopped gracefully");
	}
	
	
	
	if(CORE!=NULL)delete CORE;
	if(LOGGER!=NULL)delete LOGGER;	
	
	return 0;
}





























