/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-09-2012 12:05:18am
Filepath: includes/Agent.cpp
Lines of code: 130

Description: 

***********************************************************/


Agent::Agent(ServerCore * core,string name)
{
	m_core = core;	
	m_name = name;
	
	m_config = core->m_config;  //just a shortcut to shorten access code
	
	DoDump2Disk=false;
	m_sentheartbeatalert=false;
	
	//hard coded defaults:
	m_pollinterval=60;
	m_failheartbeat=100; 
	
	m_lastupdate=DateTime::Now();
}

Agent::~Agent()
{
	Monitor * tmpmon=NULL;
	while(m_monitors.PopFrom(tmpmon,0))
	{		
		delete tmpmon;
	}
}

void Agent::Log(int value,string text)
{
	if(m_core!=NULL)m_core->Log(value,"Agent["+m_name+"] " + text);		
}


bool Agent::Load()
{	
	if(m_core==NULL || m_config==NULL)
	{
		Log(0,"Agent::Load(): Error: Agent.m_core or Agent.m_config is NULL");
		return false;
	}
	
	Log(3,"Loading agent settings...");	
	
	m_originip=m_config->GetValueStr("agents." + m_name + ".originip");
	m_pollinterval=m_config->GetValueInt("agents." + m_name + ".pollinterval");
	m_failheartbeat=m_config->GetValueInt("agents." + m_name + ".failheartbeat");
	
	
	if(m_config->ValueExists("agents." + m_name + ".alert_emails"))
	{
		m_alert_emails = Parser::Explode(m_config->GetValueStr("agents." + m_name + ".alert_emails"), ";");
	}
	
	
	Log(3,"Loading Monitors...");
	
	xVector<string> monitorsstr = m_config->GetBlocks("agents." + m_name);
	
	string tmpstr;
	Monitor * tmpmon;
	
	for(int i=0;monitorsstr.Get(tmpstr,i);i++)
	{		
		if(m_config->GetValueInt("agents." + m_name + "." + tmpstr + ".enabled")==1)
		{
			Log(4,"Monitor: " + tmpstr + "...");
			tmpmon = new Monitor(m_core,this,tmpstr);
			tmpmon->Load("agents." + m_name,m_config);		
			m_monitors.Add(tmpmon);
		}
		else
		{
			Log(4,"Monitor: " + tmpstr + "...Not loading - disabled");
		}
	}
	
	return true;	
}



bool Agent::Poll()
{
	Log(7,"Core::Poll() monitors");
	
	if(DoDump2Disk)
	{
		mkdir(("output/"+m_name).c_str(),S_IRWXU);
	}
	
	Monitor * tmpmon=NULL;
	for(int i=0;m_monitors.Get(tmpmon,i);i++)
	{
		tmpmon->Poll();
		
		if(DoDump2Disk)
		{
			tmpmon->Dump2Disk();			
		}
	}	
	
	DoDump2Disk=false;
	
	
	
	if(!m_sentheartbeatalert && (m_lastupdate+TimeSpan::FromSeconds(m_failheartbeat))<DateTime::Now())
	{
		m_sentheartbeatalert=true;		
		FireAlarm();
	}
	
	
	
	return true;
}


bool Agent::Dump2Disk()
{
	
	Monitor * tmpmon=NULL;
	for(int i=0;m_monitors.Get(tmpmon,i);i++)
	{		
		tmpmon->Dump2Disk();
	}	
	
	return true;
}



Monitor * Agent::GetMonitor(string name)
{
	Monitor * tmpmon=NULL;
	for(int i=0;m_monitors.Get(tmpmon,i);i++)
	{		
		if(tmpmon->m_name==name)return tmpmon;
	}	
	return NULL;
}


void Agent::HeartBeat()
{
	m_lastupdate.SetNow();	
	
	if(m_sentheartbeatalert)
	{
		FireAlarm(true);
	}
	m_sentheartbeatalert=false;//reset
}



void Agent::FireAlarm(bool recover)
{
	stringstream alertss;	
	alertss<<"       Host: "<<m_name<<"\r\n       Date: "<<DateTime::Now().GetDate()<<"\r\nLast Update: "<<m_lastupdate.GetDate()<<"\r\nDescription: Heartbeat not received";	
	
	
	string emailbody = alertss.str();
	
	string emailsubject = "*ALARM* " + m_name + ": Heartbeat not received" + (recover? " [Recover]" : "");
	
	
	Log(2,"fire alert: " + emailsubject);	
	
	if(m_config->ValueExists("smtpserver"))
	{
		string smtpserver = m_config->GetValueStr("smtpserver");
		string emailfrom = m_config->GetValueStr("emailfrom");
		
		string email;
		for(int i=0;m_alert_emails.Get(email,i);i++)
		{
			EmailClient client(smtpserver);		
			
			if(emailfrom!="")client.SetFrom(emailfrom);
			
			client.SendTo(email,emailsubject,emailbody);
			Log(4,"sent email alert to " + email);
		}
	}
	
}