/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-09-2012 12:00:55am
Filepath: includes/AgentNetSession.cpp
Lines of code: 229

Description: 

***********************************************************/


AgentNetSession::AgentNetSession(ServerCore * core,ServerClient client)
{
	m_core = core;
	m_client = client;
	m_handlernum=0;
	m_subhandlernum=0;
	
	m_agentobject=NULL;
	m_monitor=NULL;
	
	Log(2,"Agent connected");
}


AgentNetSession::~AgentNetSession()
{
	Log(2,"Agent disconnected");
}


void AgentNetSession::Log(int val,string text)
{
	if(m_core!=NULL)m_core->Log(val,m_client.GetIPstr(true) + ": " + text);
}


string AgentNetSession::PopBufferStr()
{
	size_t pos = m_buffer.find("\r\n");
	
	if(pos==string::npos)return "";
	
	string ret = m_buffer.substr(0,pos);	
	m_buffer.erase(0,pos+2);
	
	
	return ret;
}



bool AgentNetSession::Poll()
{

	
	
	if(m_handlernum==0)//send connection welcome message before anything else happens
	{
		m_client.TXdata("ENVIROSTATE " + string(VERSION) + "\r\n");
		m_handlernum=2;
	}
	else
	{
		m_client.RXdata();		
		string tmpstr;
		m_client.PopData(tmpstr,m_client.QueueSize());		
		m_buffer+=tmpstr;
		
		string poppedstr = PopBufferStr();		//a string sent by the client - to process		

	
		if(poppedstr!="")
		{
			Log(6,"buffer: " + HexDump(m_buffer));
			Log(5,"poppedstr: " + HexDump(poppedstr));
			Log(5,"Handler: " + BitStream::Int2Str(m_handlernum));			
			
			switch(m_handlernum)
			{
				case 1:	//load agent
				{					
					m_agentobject = m_core->GetAgent(poppedstr);
					
					//TODO: IP check   (only certain IPs can access all agents, and also  Ip of agent can access it's own)
					
					if(m_agentobject!=NULL)
					{
						m_handlernum=2;	
						m_client.TXdata("ok\r\n");
					}
					else
					{
						m_client.TXdata("nk\r\n");						
					}					
					
				}break;
				case 2:	//command handler
				{
					string cmd=poppedstr;					
					Log(3,"command: " + cmd);
					
					if(cmd=="disc")	//disconnect
					{						
						m_client.TXdata("Bye!\r\n");	
						m_client.Disconnect();
						Log(4,"Disconnecting");
					}
					else if(cmd=="mons")	//new agent connection, in need of monitor list
					{
						if(m_agentobject==NULL)
						{
							m_client.TXdata("nk\r\n");
							m_handlernum=2;
							Log(4,"mons invalid command: m_agentobject is null");
							break;
						}
						
						m_handlernum=10;	//to agent list dump handler
						
						m_client.TXdata("ok\r\n");
						
						//store monitors in a separate array so we can pop them out one cycle at a time
						
						Monitor * tmpmon=NULL;
						
						for(int i=0;m_agentobject->m_monitors.Get(tmpmon,i);i++)
						{
							m_agentmonitors.Add(tmpmon);
						}
						
						Log(4,"internal mons list generated, ready to give to client");						
					}
					else if(cmd=="lagt")	//load agent
					{
						m_handlernum=1;
						m_client.TXdata("ok\r\n");
					}
					else if(cmd=="lmon")	//load monitor
					{
						if(m_agentobject==NULL)
						{
							m_client.TXdata("nk\r\n");
							m_handlernum=2;			
							Log(4,"lmon invalid here, m_agentobject is null");
							break;
						}
						else
						{						
							m_client.TXdata("ok\r\n");
							m_handlernum=20;
						}
					}
					else if(cmd=="wval")	//write value to a previously loaded monitor
					{
						if(m_agentobject==NULL || m_monitor==NULL || m_client.GetIPstr(false)!=m_agentobject->m_originip)
						{
							m_client.TXdata("nk\r\n");
							m_handlernum=2;	
							Log(4,"wval invalid here, (m_agentobject==NULL || m_monitor==NULL || m_client.GetIPstr(false)!=m_agentobject->m_originip) is true");
							break;
						}
						else
						{
							m_client.TXdata("ok\r\n");
							m_handlernum=21;
							m_subhandlernum=0;
						}
					}
					else if(cmd=="wfin")	//consider writes final for this monitor (sets m_value_is_final back to true)
					{
						if(m_agentobject==NULL || m_monitor==NULL || m_client.GetIPstr(false)!=m_agentobject->m_originip)
						{
							m_client.TXdata("nk\r\n");
							m_handlernum=2;	
							Log(4,"wval invalid here, (m_agentobject==NULL || m_monitor==NULL || m_client.GetIPstr(false)!=m_agentobject->m_originip) is true");
							break;
						}
						else
						{
							m_client.TXdata("ok\r\n");
							
							m_monitor->m_value_is_final=true;							
							m_agentobject->HeartBeat();
						}
					}					
					else if(cmd=="rval")	//read value from a previously loaded monitor
					{
						if(m_agentobject==NULL || m_monitor==NULL)
						{
							m_client.TXdata("nk\r\n");
							m_handlernum=2;		
							Log(4,"rval invalid here, (m_agentobject==NULL || m_monitor==NULL) is true");
							break;
						}
						else
						{
							m_client.TXdata(BitStream::Int2Str(m_monitor->GetValue()) + "\r\n");
							m_handlernum=2;
						}
					}					
					else
					{
						Log(4,"Invalid command");
						m_client.TXdata("nk\r\n");
						m_handlernum=2;
					}					
					
				}break;
				case 10:
				{
					if(poppedstr!="n")	//client must send this to get first and next item
					{
						m_handlernum=2;
						break;
					}
					
					Monitor * tmpmon=NULL;
					m_agentmonitors.PopFrom(tmpmon,0);
					
					if(tmpmon!=NULL)
					{
						//list agent
						m_client.TXdata("i\r\n");		//indicates there is an agent comming next 
						
						m_client.TXdata(tmpmon->m_name + "\r\n");									//name
						m_client.TXdata(BitStream::Int2Str(tmpmon->m_pollinterval) + "\r\n");		//interval
						m_client.TXdata(tmpmon->m_source + "\r\n");								//command to execute
						m_client.TXdata(Float2Str(tmpmon->m_multiplier) + "\r\n");					//multiplier						
					}
					else
					{
						Log(4,"agent list done");
						m_client.TXdata("e\r\n");		//indicates there are no more agents... back to command prompt
						m_handlernum=2;					//back to command handler						
					}
				}break;
				case 20:	//load monitor - get monitor name
				{					
					m_monitor = m_agentobject->GetMonitor(poppedstr);
					
					if(m_monitor==NULL)
					{
						m_client.TXdata("nk\r\n");
						Log(4,"invalid, m_monitor is null");
					}
					else
					{
						m_client.TXdata("ok\r\n");
					}
					
					m_handlernum=2;		//back to command mode - with a monitor loaded we can perform a command on it
				}break;
				case 21:	//write value: value
				{
					if(m_monitor==NULL)
					{
						m_client.TXdata("nk\r\n");
						Log(4,"invalid, m_monitor is null");
					}
					else
					{
						m_tempstr1=poppedstr;	//store value for now, as we still need the date
						
						m_handlernum=22;
					}
				}break;
				case 22:	//write value: date
				{
					m_monitor->SetValue(atoi(m_tempstr1.c_str()),atoi(poppedstr.c_str()));	
					
					m_monitor->m_value_is_final=false; //if we have a list of values to write, prevents an alarm from being triggered until all the list is done.  Important for client to send wfin after writing some values
					
					m_client.TXdata("ok\r\n");
					m_handlernum=2;
				}
			}
		}
	}
	
	
	return true;
}


bool AgentNetSession::Disconnected()
{
	return !m_client.IsConnected();
}