/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Aug-06-2012 04:08:18pm
Filepath: includes/AppLogger.cpp
Lines of code: 115

Description: 

***********************************************************/


AppLogger::AppLogger(string logfile)
{
	#ifdef rslib_pthread
		AutoFlush=false;
	#else
		AutoFlush=true;
	#endif
	
	
	ConsoleOut=false;
	DebugLevel=0;	
	
	m_logfile=logfile;
}

AppLogger::~AppLogger()
{
	Flush();
}



void AppLogger::Log(int debuglevel, string entry)
{
	if(debuglevel>DebugLevel)return;
	#ifdef rslib_pthread
		loggermutex.Lock();
	#endif
	
	Log_nomutex(debuglevel,entry);
	
	#ifdef rslib_pthread
		loggermutex.UnLock();
	#endif
	
	if(AutoFlush)Flush();
}

void AppLogger::Log_nomutex(int debuglevel, string entry)
{
	if(debuglevel>DebugLevel)return;
	
	rslib::DateTime tmpdate = rslib::DateTime::Now();	
	
	stringstream ss;	
	
	ss<<"["<<tmpdate.GetDate()<<"] ";
	
	
	ss<<"("<<debuglevel<<") ";
	
	
	for(int i=debuglevel>5? 5 : debuglevel; i>1;i--)
	{
		ss<<"   ";
	}
	ss<<entry;	
	
	string ssstr = ss.str();
	
	m_logbuffer.Add(ssstr);
	
	if(ConsoleOut || debuglevel==0)cout<<ssstr<<endl;
}


void AppLogger::Flush()
{	
	#ifdef rslib_pthread
		loggermutex.Lock();
	#endif
	
	
	if(m_logbuffer.GetSize()>0)
	{
	
		string filename=m_logfile + ".log";
				
		
		fstream fout(filename.c_str(),ios::app | ios::out);		
		
		if(!fout.is_open())
		{
			cout<<"AppLogger::Flush() : Error opening log file for writing";
		}
		else
		{
			try
			{
				if(!AutoFlush) fout<<"."; //to indicate flush point
				string logentry;
				for(int i=0;m_logbuffer.Get(logentry,i);i++)
				{		
					fout<<logentry<<"\n";	
				}
				
				m_logbuffer.Clear();
				
				fout.close();  
			}
			catch(string e)
			{
				cout<<"AppLogger::Flush() : Exception caught";
			}
		}
	}
	
	#ifdef rslib_pthread
		loggermutex.UnLock();
	#endif
}



void AppLogger::CheckRotation(unsigned long int maxlines,bool force)
{
	#ifdef rslib_pthread
		loggermutex.Lock();
	#endif	
	
	bool dorotate=false;
	
	if(force)
	{
		dorotate=true;
	}
	else
	{
		//count lines of current log file
		
		fstream fin((m_logfile + ".log").c_str(),ios::in | ios::binary);
		
		if(fin)
		{		
			for(unsigned long int i=0;!fin.eof();i++)
			{
				for(char data=' ';!fin.eof() && data!='\n'; data=fin.get()); //get line
				
				if(i>=maxlines)
				{
					dorotate=true;
					break;
				}
			}
		}
		
		fin.close();
	}

	
	if(dorotate)
	{
		unlink((m_logfile + "-old.log").c_str());
		rename((m_logfile + ".log").c_str(),( m_logfile + "-old.log").c_str());
	}
	
	#ifdef rslib_pthread
		loggermutex.UnLock();
	#endif
}
