/**********************************************************
HVACCTL C++ header file
Home HVAC Control System
Last modified by Ryan Auclair on Feb-15-2012 09:46:36pm
Filepath: includes/AppLogger.h
Lines of code: 23

Description: Universal logging class to generate program logs for debugging and info viewing

***********************************************************/

/**
How to use this class:

Declare a logger as a global for the program.
Use said logger to log various activities throughout the program. 
Ensure to call flush occasionally, or set autoflush to true (default if in single thread mode)

logger statuses:

0: Assert: This will deplay to the console all the time, used for situations that should not happen
1: General info such as startup/shutdown as well as errors.  This mode should normally be set to default

2-5: various levels of logging based on application

**/



#ifndef logger_h
#define logger_h

#include <fstream>

class AppLogger
{	
	private:
		xVector<string> m_logbuffer;
		string m_logfile;	
		
	
	public:
		AppLogger(string logfile); //specify log file name without .log	
		~AppLogger(); 	//specify log file name without .log	
		bool ConsoleOut;	//Output to console also
		bool AutoFlush;	//Flush buffer after each entry
		int DebugLevel;	//Debug level to show/write
		
		#ifdef rslib_pthread
			MUTEX loggermutex;	
		#endif
		
		void Log(int level,string entry);					//generates log entry
		void Log_nomutex(int level,string entry);				//generates log entry without checking mutex (caution)
		void Flush();								//flushes buffer to file
		void CheckRotation(unsigned long int maxlines,bool force=false); 	//checks if file should be rotated based on given max lines

};



#endif