/**********************************************************
EnviroState C++ header file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-08-2012 02:11:19am
Filepath: includes/Core/AgentCore.h
Lines of code: 43

Description: Core for agent mode

***********************************************************/


#ifndef agentcore_h
#define agentcore_h

class Monitor;
class AppLogger;
class MlConfFile;
class DataEntry;
class MonitorEntry;

class AgentCore : public Core
{
	private:
		ClArgs m_args;
		AppLogger * m_logger;
		MlConfFile * m_config;
		SocketClient * m_tcpclient;
		xVector<MonitorEntry*> m_monitorentries;
		
		time_t m_lastmonret;		//last time monitors were retrieved
		time_t m_lastdatasend;		//last time we sent data
		time_t m_lastconattempt;	//last time we tried to reconnect (ex: if it failed)
		time_t m_lastpidcheck;		//last time pid file was checked
		bool m_serverdown;			//if server has been deemed down
		
		string m_tcpbuffer;
		
		//------- config settings ---------
		//---------------------------------
		
		string m_agentname;
		string m_serverhost;
		int m_serverport;
		int m_datasendinterval;
		
		//------- end config settings ------
		//----------------------------------	
		

		bool TcpSend(string tosend);	//send a string
		string TcpRecv();				//receive a string
		
		
		bool CheckConnection();		//checks if we are connected, if not attempts to connect, or returns false if server is down		
		bool Connect();				//establish tcp connection to server
		bool IsConnected();			//wether it is connected or not
		void ClearMonList();		//clears monitor list (need to do this before fetching new monitor list)
		void Disconnect();			//disconnects tcp connection
		bool RecvMonitors();		//receive list of monitors from server
		bool SendMonitors();		//sends data to server		
		
	public:		
		AgentCore(ClArgs args,AppLogger * logger);
		~AgentCore();
		void Log(int level,string text);
		bool Load();		//perform initial tcp connection, download monitor info, etc
		void Terminate();	//terminate core
		bool Poll();		//poll monitors
		

	
};




#endif