/**********************************************************
EnviroState C++ header file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-03-2012 05:58:35pm
Filepath: includes/Core/ServerCore.h
Lines of code: 35

Description:  Core of the application when running in server mode

***********************************************************/


#ifndef servercore_h
#define servercore_h

class Monitor;
class Agent;
class AppLogger;
class MlConfFile;
class AgentNetSession;

#include "Core.h"

class ServerCore : public Core
{
	friend class Monitor;
	friend class Agent;

	private:
		ClArgs m_args;
		AppLogger * m_logger;
		MlConfFile * m_config;
		bool m_server;	
		SocketServer m_serverclass;
		DateTime m_lastdatadump;		
		time_t m_lastpidcheck;
		xVector<AgentNetSession*> m_agentsessions;
		
		//------- config settings ---------
		//---------------------------------
			
		int m_datadumpinterval;
		int m_listenport;
		
		xVector<Agent*> m_agents;
		
		Monitor * m_defaultmonitor;
			
		//------- end config settings ------
		//----------------------------------

  public:
		ServerCore(ClArgs args,AppLogger * logger);
		~ServerCore();
		void Log(int level,string text);

		bool Load();		//loads config and does other init stuff, returns false on failure
		void Terminate();	//ensures all data is written to disk etc - note: freeing memory is up to the destructor
		
		bool Poll();		//polls all monitors, returns false if we received a signal to stop application

		Agent * GetAgent(string name);	//gets agent by name
		




};

#endif
