/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Sep-28-2012 01:03:59am
Filepath: includes/MlConfFile/MlConfBlock.cpp
Lines of code: 340

Description: 

***********************************************************/


MlConfBlock::MlConfBlock(MlConfFile * conffile,MlConfBlock * parent,string name)
{
	m_mlconffile=conffile;
	m_parent=parent;
	m_name=name;
}


MlConfBlock::~MlConfBlock()
{
	Log(4,"Destructor called");
	
	MlConfBlock * tmp=NULL;
	while(m_blocks.PopFrom(tmp,0))
	{
		if(tmp==NULL)continue;
		Log(4,"Freeing blc " + tmp->Name() + "...");
		delete tmp;
		tmp=NULL;
	}

	MlConfVar * tmp2=NULL;
	while(m_vars.PopFrom(tmp2,0))
	{
		if(tmp2==NULL)continue;
		Log(4,"Freeing var " + tmp2->Name() + "...");
		delete tmp2;
		tmp2=NULL;
	}
}


void MlConfBlock::Log(int level,string data)
{	
	if(m_parent==NULL)
	{
		if(m_mlconffile!=NULL)m_mlconffile->Log(level,m_name + ":" + data);
		else return;
	}
	else
	{	
		m_parent->Log(level,m_name + ":" + data);
	}
}



string MlConfBlock::Name()
{
	return m_name;
}


bool MlConfBlock::Load(fstream & fd,string filepath)
{
	Log(3,"Reading for keyword...");
	
	//start by searching for a keyword, while ignoring anything else such as comments

	char buf[3]={'X','Y','Z'}; //used to keep track of 3 bytes back (sometimes need to backreference)
	string tmp1,tmp2,tmp3;
	char ch='C';

	int tmpint;
	float tmpfloat;
	MlConfBlock * tmpblock=NULL;

	int status=0;
	int substatus=0;
	/*
	status:
	0=normal read
	1=reading variable and value
	2=reading block
	3=reading include
	*/


	while(!fd.eof())
	{
		ch = fd.get();
		
		if(ch=='#') //comment, ignore everything till return is found
		{
			while(!fd.eof())
			{
				ch=fd.get();
				if(ch=='\r' || ch=='\n')break;
			}
			//set status back to 0, that way if a comment is thrown in the middle of a variable declaration or something, the parser will be forgiving and just ignore that line
			status=0;

			continue;
		}
		
		//rogue returns and white spaces are ignored 	  
		if(ch=='\r' || ch=='\n' || ch==' ' || ch==0x9)continue;
		
		
		string debugtmp;
		debugtmp.append(1,ch);
		Log(6,"char: '" + debugtmp + "'");
		
		
		//backfill small buffer with valid char:
		//---
		
		buf[0]=buf[1];
		buf[1]=buf[2];	  
		buf[2] = ch;	  
		
		//check if we got a keyword or something to work with:
		//---
		
		if(status==0 && buf[0]=='s' && buf[1]=='t' && buf[2]=='r')   //str variable declaration
		{
			status=1;
			tmp1="str";
			tmp2="";
			tmp3="";
			
			Log(4,"Found variable keyword: str");    
			continue;
		}
		
		if(status==0 && buf[0]=='i' && buf[1]=='n' && buf[2]=='t')   //int variable declaration
		{
			status=1;
			tmp1="int";
			tmp2="";
			tmp3="";
			
			Log(4,"Found variable keyword: int");    
			continue;
		}
		
		if(status==0 && buf[0]=='f' && buf[1]=='l' && buf[2]=='o')   //flo variable declaration
		{
			status=1;
			tmp1="flo";
			tmp2="";
			tmp3="";
			
			Log(4,"Found variable keyword: flo");    
			continue;
		}
		
		
		
		if(status==0 && (buf[0]=='b' && buf[1]=='l' && buf[2]=='c'))
		{
			status=2;
			substatus=0;
			tmp1="";
			tmp2="";
			tmp3="";
			
			Log(4,"Found block keyword: blc");
			continue;
		}
		
		if(status==0 &&  (buf[0]=='i' && buf[1]=='n' && buf[2]=='c'))
		{
			status=3;
			substatus=0;
			tmp1="";
			tmp2="";
			tmp3="";		
			
			Log(4,"Found include keyword: inc");
		}
		
		if(status==0 && ch=='}') 
		{
			Log(4,"End of block");
			
			return true;
		}
		
		
		
		if(status==1)
		{
			if(ch=='=') //done getting var name now look for quotes and read value
			{
				Log(4,"Found equal sign, now looking for first quote");
				substatus=0;
				while(!fd.eof() && status==1)
				{
					ch=fd.get();
					buf[0]=buf[1];
					buf[1]=ch;
					
					if(substatus==0 && ch=='"') //found first quote
					{
						Log(4,"Found first quote");
						substatus=1;
						continue;					
					}
					
					if(substatus==1)
					{
						if(ch=='\\') //escape char
						{
							Log(4,"Found escape char");
							ch=fd.get();
							
							if(ch=='\\')tmp3.append(1,'\\');
							if(ch=='"')tmp3.append(1,'"');						
						}
						else if(ch=='"') //end of value
						{
							Log(4,"Found last quote, now got value of \"" + tmp3 + "\"");
							//variable and value gotten, create and add to array:
							
							if(tmp1=="str")
							{
								SetValue(new MlConfVar(tmp2,(string)tmp3));
							}
							else if(tmp1=="int")
							{							
								SetValue(new MlConfVar(tmp2,(int)atol(tmp3.c_str())));
							}
							else if(tmp1=="flo")
							{							
								SetValue(new MlConfVar(tmp2,(float)atof(tmp3.c_str())));
							}
							
							status=0;	
							substatus=0;
						}
						else //keep reading
						{
							tmp3.append(1,ch);
						}
					}
				}
						
			}
			else
			{
				tmp2.append(1,ch);  //read var name
			}
			
			continue;
		}
		
		
		if(status==2)
		{
			if(ch=='{') //done getting block name, next read it
			{
				
				if(tmp1!="")
				{				
					//pass on to recursive call....
					MlConfBlock * addblock = new MlConfBlock(m_mlconffile,this,tmp1);
					addblock->Load(fd,filepath);
					AddConfBlock(addblock);
				}
				else
				{
					Log(1,"Block name blank, skipping");
					return false;
				}
				
				status=0;
				continue;
			}
			else
			{
				tmp1.append(1,ch);
			}			
		}
		
		if(status==3)
		{
			substatus=0;
			Log(5,"reading include file block name");
			while(!fd.eof() && status==3)
			{
				ch=fd.get();
				
				
				if(substatus==0)
				{
					 if(ch=='\r' || ch=='\n' || ch==' ' || ch==0x9)continue; //ignore spaces and returns
					 
					 if(ch=='"')
					 {
						substatus=1;
						Log(5,"reading include file name");
						continue;
					 }
					 
					 tmp1.append(1,ch);
				}
				
				if(substatus==1)
				{
					if(ch=='"')
					{
						Log(5,"Ready to start loading from included file.  Block: " + tmp1 + ", File: " + tmp2);
						
						MlConfBlock * fileblock = new MlConfBlock(m_mlconffile,this,tmp1);						
						m_mlconffile->Load(fileblock,tmp2);
						AddConfBlock(fileblock);
					
						status=0;
					}
					else
					{
						tmp2.append(1,ch);
					}
				}
			}
		}
	}


	return true;
}



MlConfVar * MlConfBlock::GetValue(string var)
{
	Log(3,"Get value for: " + var);
	
	//var will look something like this:  block.block2.var or perhaps just var.  Need to parse out block and call it from appropriate block

	MlConfVar * ret=NULL;
	
	int pos=0;
	
	pos = var.find(".");
	
	if(pos!=string::npos && pos<var.size())
	{
		//fetch subblock portion:		
		string subblock=var.substr(0,pos);
		
		Log(5,"Subblock: " + subblock);
		
		//find it:
		MlConfBlock * tmp=NULL;
		for(int i=0;m_blocks.Get(tmp,i);i++)
		{
			if(tmp->Name()==subblock)return tmp->GetValue(var.substr(pos+1,var.size()-pos-1));
		}
		
		Log(1,"Block not found");
		
		//block not found, returning null	
		
		return 	ret;
	}
		
	//looking at a variable, find it and return value:
	
	Log(1,"Variable: " + var);

	//find it:
	MlConfVar * tmp2=NULL;
	for(int i=0;m_vars.Get(tmp2,i);i++)
	{
		if(tmp2->Name()==var)return tmp2;
	}
	

	return ret;
}


MlConfBlock * MlConfBlock::GetBlock(string path)
{
	Log(4,"Getting block at path: " + path);
	xVector<string> blocks = Parser::Explode(path,".");	

	string firstblockname;
	if(!blocks.PopFrom(firstblockname,0))return NULL;
	
	
	MlConfBlock * blk=NULL;
	
	for(int i=0;m_blocks.Get(blk,i);i++)
	{
		if(blk->Name()==firstblockname)
		{
			if(blocks.GetSize()>0)
			{
				Log(5,"Moving on to next block in path");
				path.erase(0,firstblockname.size()+1);
				return blk->GetBlock(path);
			}
			else
			{
				Log(5,"Returning block: " + blk->Name());
				return blk;
			}
		}
	}
	
	Log(5,"Invalid path/block");
	
	//if we made it here means there is no valid block so we return null
	return NULL;
}


void MlConfBlock::SetValue(MlConfVar * var)
{
	Log(4,"Setting var " + var->Name());
	
	m_vars.Add(var);
}

void MlConfBlock::AddConfBlock(MlConfBlock * block)
{
	Log(4,"Adding block " + block->Name());
	
	m_blocks.Add(block);
}


string MlConfBlock::DumpData()
{
	return DumpData("");
}


string MlConfBlock::DumpData(string base)
{
	string ret;
	
	if(base!="")base+=".";
	base+=m_name;
	
	
	MlConfVar * tmp2=NULL;
	for(int i=0;m_vars.Get(tmp2,i);i++)
	{
		ret+= tmp2->TypeStr() + " " + base + "." + tmp2->Name() + " = \"" + tmp2->GetStr() + "\"\r\n";	
	}
	

	
	//iterate through blocks:
	MlConfBlock * tmp=NULL;
	for(int i=0;m_blocks.Get(tmp,i);i++)
	{
		ret+=tmp->DumpData(base);
	}


	return ret;
}



xVector<string> MlConfBlock::GetBlocks(string path)
{
	xVector<string> ret;

	MlConfBlock * endblock = GetBlock(path);
	if(endblock==NULL)return ret;
	
	Log(4,"Getting list of blocks in: " + path);
	
	MlConfBlock * tmp=NULL;
	for(int i=0;endblock->m_blocks.Get(tmp,i);i++)
	{
		Log(5,"Adding to list: " + tmp->Name());
		ret.Add(tmp->Name());
	}


	return ret;
}

