/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-08-2012 09:50:42pm
Filepath: includes/MlConfFile/MlConfFile.cpp
Lines of code: 117

Description: 

***********************************************************/


MlConfFile::MlConfFile(string fullfilename,AppLogger * LOGGER)
{
  SplitPathFile(fullfilename,m_filepath,m_filename);
  
  m_LOGGER=LOGGER;
  DebugLevel=0;
  
  m_root = new MlConfBlock(this,NULL,"ROOT");
}


MlConfFile::~MlConfFile()
{
	Log(5,"Freeing all memory...");
	delete m_root;
	Log(5,"Done freeing all memory...");
}


void MlConfFile::Log(int level,string data)
{
	if(m_LOGGER==NULL || level>DebugLevel)return;
	
	m_LOGGER->Log(level,"mlconf[" + m_filepath + m_filename + "] " + data);
}



void MlConfFile::SplitPathFile(string fullpath, string & path, string & file)
{
	path="";
	file="";
	
	xVector<string> parts = Parser::Explode(fullpath,"/");

	parts.PopFrom(file,parts.GetSize()-1);

	string tmp;	
	for(int i=0;parts.Get(tmp,i);i++)
	{
		path+=tmp+"/";
	}
}



bool MlConfFile::Load()
{
	Log(1,"Loading...");
	bool ret=Load(m_root,m_filename);
	if(!ret)Log(0,"Aborted loading due to error(s) Please check config file syntax");
	else Log(1,"Done loading");
	
	return ret;
}


bool MlConfFile::Load(MlConfBlock * curblock, string file)
{
	bool ret=true;
	
	Log(2,"Loading file: " + file + " (in " + m_filepath + ")");
	
	//for security reasons includes cannot have ../ or start with / this is to restrict refering to files outside of the main config folder:
	
	if(curblock!=m_root)  
	{
		if(file.size()>0 && file[0]=='/')ret=false;
		else if(file.find("../")!=string::npos)ret=false;
		
		if(!ret)
		{
			Log(0,"Illegal filename, returning false");
			return false;	
		}
	}
	
	string fullfile=m_filepath+file;
	
	//make sure this file was not already loaded:       (could mean a recursive call that could send the loader in an infinite loop)
	
	string tmp;
	for(int i=0;m_loadingfiles.Get(tmp,i);i++)
	{
		if(tmp==fullfile)
		{
			Log(0,"Possible recursive or reload of same file, returning false");
			return false;
		}	
	}
	
	//ok we made it here, the path is valid!  We'll add it to the array...
	m_loadingfiles.Add(fullfile);
	
	
	//and now we can finally start loading:
	
	fstream fd(fullfile.c_str(),ios::in);
	
	if(!fd) //oops, file not found... return false and skip this one
	{
		Log(0,"File not found");
		return false;
	}
		
	string filepath;	
	SplitPathFile(fullfile,filepath,tmp);
		
	ret= curblock->Load(fd,filepath);
	fd.close();
	
	return ret;
}


MlConfVar * MlConfFile::GetValue(string str)
{
	return m_root->GetValue(str);
}


bool MlConfFile::BlockExists(string path)
{
	MlConfBlock * tmp = m_root->GetBlock(path);
	
	return (tmp!=NULL);
}

bool MlConfFile::ValueExists(string path)
{
	MlConfVar * tmp = m_root->GetValue(path);
	
	return (tmp!=NULL);
}





string MlConfFile::GetValueStr(string settingpath)
{
  MlConfVar * retval = GetValue(settingpath);
  
  if(retval!=NULL)return retval->GetStr();  
  
  return "";
}



int MlConfFile::GetValueInt(string settingpath)	
{
  MlConfVar * retval = GetValue(settingpath);
  
  if(retval!=NULL)return retval->GetInt();  
  
  return 0;
}



float MlConfFile::GetValueFlo(string settingpath)
{
  MlConfVar * retval = GetValue(settingpath);
  
  if(retval!=NULL)return retval->GetFlo();  
  
  return 0;
}



string MlConfFile::DumpData()
{
	if(m_root==NULL)return "null";
	
	return m_root->DumpData();
}


xVector<string> MlConfFile::GetBlocks(string path)
{
	if(m_root==NULL)return xVector<string>();
	
	return m_root->GetBlocks(path);
}