/**********************************************************
EnviroState C++ header file
Environmental Monitoring System
Last modified by Red Squirrel on Sep-08-2012 08:27:43pm
Filepath: includes/MlConfFile/MlConfFile.h
Lines of code: 32

Description: This is the main file for the Multi Level Config File class.
This class is made to load a config file and store it's values for
easy retrieval in a program.  These config files are meant
to be edited by hand by the user so they are read only.

***********************************************************/


/***********************************************
Config file format:

#a comment

int varint="3"  
str varstring="a string"
flo varfloat = "3.4"

blc block
{
#can add variables or other blocks here
}

#include another file as a block
inc blockname "afile.conf"


to retrive a variable, the format block.var is used. 

Multi block vars are simply acessed as block1.block2.block3.varname

 
************************************************/



#ifndef MlConfFile_h
#define MlConfFile_h


class MlConfBlock;
class MlConfVar;


class MlConfFile
{
  friend class MlConfBlock;
  friend class MlConfVar;
  
  private:
    string m_filepath; 			//path of file name with trailing slash
    string m_filename; 			//filename (without path)
    AppLogger * m_LOGGER;
	
	xVector<string> m_loadingfiles; //used during loading process to keep track of files being loaded
    
   //conf file data
   MlConfBlock * m_root;		//root block - all blocks are put in here  

   MlConfVar * GetValue(string);		//gets value as a MlConfVar  (called by the other getvalue functions)
  
  
    bool Load(MlConfBlock * curblock, string fullfile);				//loads blocks/variables from specific path/file
  
	
	void Log(int level,string data);	//debug output
  public:
    MlConfFile(string fullfilename,AppLogger *);
    ~MlConfFile();

	int DebugLevel;

	bool Load();	//load entire file											
    
    	
	bool BlockExists(string path);	//see if a certain block exists
	bool ValueExists(string path);	//see if a certain value exists
	
	//get values:
    //ex path:   level1.level2.varname
    //ex cont.   varname  (if it's in root)
    
    
	string GetValueStr(string path);		//gets value as a string
	int GetValueInt(string path);		//gets value as an int
	float GetValueFlo(string path);		//gets value as float
	
	string DumpData();				//dumps entire config file into a string for displaying    
    xVector<string> GetBlocks(string path);	//returns a list of blocks within a specified block
    
    
    
    static void SplitPathFile(string tosplit, string & path, string & file);	//splits a full path to the folder path and file
};


#endif