/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-12-2012 12:19:18am
Filepath: includes/Monitor.cpp
Lines of code: 298

Description: 

***********************************************************/




Monitor::Monitor(ServerCore * core, Agent * agent, string name)
{
	m_core = core;
	m_agent = agent;
	m_name=name;
	m_agentname=(agent!=NULL ? agent->m_name : "");
	m_alarmstate=0;
	
	m_value_is_final=false;
	m_virgin=true;
	m_lastvalue=0;
	m_alarmhigh=false;
	
	
	m_ignoremin=false;
	m_ignoremax=false;
	m_enable_alm_min=true;
	m_enable_alm_maj=true;
	m_enable_alm_crit=true;
	
	
	if(m_core!=NULL && m_core->m_defaultmonitor!=NULL)
	{
		Monitor * copyfrom = m_core->m_defaultmonitor;		
		
		m_description = copyfrom->m_description;
		m_low_alarm_txt = copyfrom->m_low_alarm_txt;
		m_high_alarm_txt = copyfrom->m_high_alarm_txt;		
		m_pollinterval = copyfrom->m_pollinterval;		
		m_source = copyfrom->m_source;		
		m_sticky = copyfrom->m_sticky;
		m_stickydir = copyfrom->m_stickydir;		
		m_multiplier = copyfrom->m_multiplier;
		m_alm_min_min = copyfrom->m_alm_min_min;
		m_alm_min_max = copyfrom->m_alm_min_max;
		m_alm_maj_min = copyfrom->m_alm_maj_min;
		m_alm_maj_max = copyfrom->m_alm_maj_max;
		m_alm_crit_min = copyfrom->m_alm_crit_min;
		m_alm_crit_max = copyfrom->m_alm_crit_max;	
	}
	
	
}



Monitor::~Monitor()
{	
	DataEntry * tmpentry=NULL;
	while(m_dataentries.PopFrom(tmpentry,0))
	{	
		delete tmpentry;
	}	
}


void Monitor::Log(int val,string txt)
{
	if(m_agent!=NULL)m_agent->Log(val,"Mon["+m_name+"] " + txt);	
	else if(m_core!=NULL)m_core->Log(val,"Mon["+m_name+"] " + txt);	
}


void Monitor::Load(string confpath,MlConfFile * conf)
{
	m_conf=conf;
	
	if(confpath!="")confpath+=".";	
	confpath+=m_name+".";
	
	string tmpstr,tmpstr2;
	
	Log(3,"Loading from " + confpath + "...");
	
	if(conf->ValueExists(confpath+"description"))m_description=conf->GetValueStr(confpath+"description");
	
	
	if(conf->ValueExists(confpath+"low_alarm_txt"))m_low_alarm_txt=conf->GetValueStr(confpath+"low_alarm_txt");
	if(conf->ValueExists(confpath+"high_alarm_txt"))m_high_alarm_txt=conf->GetValueStr(confpath+"high_alarm_txt");
																					 
	if(conf->ValueExists(confpath+"pollinterval"))m_pollinterval=conf->GetValueInt(confpath+"pollinterval");
	
	if(conf->ValueExists(confpath+"source"))m_source=conf->GetValueStr(confpath+"source");
	
	if(conf->ValueExists(confpath+"sticky"))m_sticky=conf->GetValueInt(confpath+"sticky");
	if(conf->ValueExists(confpath+"stickydir"))m_sticky=conf->GetValueInt(confpath+"stickydir");
	
	if(conf->ValueExists(confpath+"multiplier"))m_multiplier=conf->GetValueFlo(confpath+"multiplier");	
	
	if(conf->ValueExists(confpath+"ignoremin"))m_ignoremin=true;	//undocumented, but left anyway
	if(conf->ValueExists(confpath+"ignoremax"))m_ignoremax=true;	//undocumented, but left anyway
	
	if(conf->BlockExists(confpath+"alm_min"))
	{
		if(conf->ValueExists(confpath+"alm_min.min"))
		{
			m_alm_min_min=conf->GetValueInt(confpath+"alm_min.min");	
		}
		else m_ignoremin=true;
		
		if(conf->ValueExists(confpath+"alm_min.max"))
		{
			m_alm_min_max=conf->GetValueInt(confpath+"alm_min.max");	
		}
		else m_ignoremax=true;
		
		m_enable_alm_min=(!m_ignoremin | !m_ignoremax);
	}
	
	if(conf->BlockExists(confpath+"alm_maj"))
	{
		if(conf->ValueExists(confpath+"alm_maj.min"))
		{
			m_alm_maj_min=conf->GetValueInt(confpath+"alm_maj.min");	
		}
		else m_ignoremin=true;
		
		if(conf->ValueExists(confpath+"alm_maj.max"))
		{
			m_alm_maj_max=conf->GetValueInt(confpath+"alm_maj.max");	
		}
		else m_ignoremax=true;
		
		m_enable_alm_maj=(!m_ignoremin | !m_ignoremax);
	}
	
	if(conf->BlockExists(confpath+"alm_crit"))
	{
		if(conf->ValueExists(confpath+"alm_crit.min"))
		{
			m_alm_crit_min=conf->GetValueInt(confpath+"alm_crit.min");	
		}
		else m_ignoremin=true;
		
		if(conf->ValueExists(confpath+"alm_crit.max"))
		{
			m_alm_crit_max=conf->GetValueInt(confpath+"alm_crit.max");	
		}
		else m_ignoremax=true;
		
		m_enable_alm_crit=(!m_ignoremin | !m_ignoremax);
	}
	
	xVector<string> tmpemails;
	
	if(conf->ValueExists(confpath+"alert_emails"))
	{
		tmpemails = Parser::Explode(conf->GetValueStr(confpath+"alert_emails"), ";");	
	}
	
	
	
	
	//add emails to list from various sources, checking for duplicates:
	//-----------------------------------------------------------------
	
	
	if(m_agent!=NULL)
	{		
		for(int i=0;m_agent->m_alert_emails.Get(tmpstr,i);i++)
		{
			tmpemails.Add(tmpstr);
		}
	}
	
	if(m_core->m_defaultmonitor!=NULL)
	{
		for(int i=0;m_core->m_defaultmonitor->m_alert_emails.Get(tmpstr,i);i++)
		{
			tmpemails.Add(tmpstr);
		}
	}
	
	
	//clear list so we can start over:
	m_alert_emails.Clear();
	
	bool exists=false;
	while(tmpemails.PopFrom(tmpstr,0))
	{
		//ensure it does not already exist:
		exists=false;
		
		for(int i=0;m_alert_emails.Get(tmpstr2,i);i++)
		{
			if(tmpstr==tmpstr2)
			{
				exists=true;
			}
		}
		
		if(exists)continue;		
		
		m_alert_emails.Add(tmpstr);
	}
	
	
}




int Monitor::GetValue()
{
	return m_lastvalue;
}


void Monitor::SetValue(int val,int date)
{
	m_lastvalue=val;
	m_virgin=false;
	
	DataEntry * toadd = new DataEntry;
	
	toadd->value=val;
	toadd->date=date;
	
	m_dataentries.Add(toadd);
}

void Monitor::SetValue(int val)
{
	m_lastvalue=val;
	m_virgin=false;
	
	DataEntry * toadd = new DataEntry;
	
	toadd->value=val;
	toadd->date=(int)time(NULL);	
	
	m_dataentries.Add(toadd);
}



bool Monitor::IntervalReached()
{
	DateTime lastplus = m_lastchecked;
	lastplus.AddSeconds(m_pollinterval);	
	
	if(DateTime::Now()>lastplus)return true;
	
	return false;
}


bool Monitor::Poll()
{
	if(!IntervalReached() || !m_value_is_final || m_virgin)return false;
	
	Log(3,"Polling...");
	m_lastchecked = DateTime::Now();
	
		
	//todo: check stickyness, also enable flip/flop prevention
	
	
	
	//check alarm status starting with critical. major and minor as worse is always what we want to throw
	
	int raisealarm=0;	//1=min, 2=maj, 3=crit
	bool high=false;	//wether alarm is high or low
	bool firerecovery=false;
	string almtext="";
	
	if(m_enable_alm_crit)
	{
		if(!m_ignoremin && m_lastvalue<m_alm_crit_min)
		{
			raisealarm=3;
			high=false;
		}
		if(!m_ignoremax && m_lastvalue>m_alm_crit_max)
		{
			raisealarm=3;
			high=true;
		}		
	}
	
	if(raisealarm==0 && m_enable_alm_maj)
	{
		if(!m_ignoremin && m_lastvalue<m_alm_maj_min)
		{
			raisealarm=2;
			high=false;
		}
		if(!m_ignoremax && m_lastvalue>m_alm_maj_max)
		{
			raisealarm=2;
			high=true;
		}		
	}
	
	if(raisealarm==0 && m_enable_alm_min)
	{
		if(!m_ignoremin && m_lastvalue<m_alm_min_min)
		{
			raisealarm=1;
			high=false;
		}
		if(!m_ignoremax && m_lastvalue>m_alm_min_max)
		{
			raisealarm=1;
			high=true;
		}		
	}
	
	
	
	
	
	stringstream debuginfoss;
	
	debuginfoss<<"Alarm determinant vars: m_lastvalue="<<m_lastvalue;
	debuginfoss<<", m_enable_alm_min="<<m_enable_alm_min<<", m_enable_alm_maj="<<m_enable_alm_maj<<", m_enable_alm_crit="<<m_enable_alm_crit;	
	debuginfoss<<", m_ignoremin="<<m_ignoremin<<", m_ignoremax="<<m_ignoremax;
	debuginfoss<<", m_alarmstate="<<m_alarmstate<<", raisealarm="<<raisealarm<<", high="<<high<<", m_alarmhigh="<<m_alarmhigh;
	
	Log(4,debuginfoss.str());
	
	//fire alarm upon state change for worse, or from high to low or vise versa
	if(m_alarmstate<raisealarm || (raisealarm>0 && high!=m_alarmhigh))
	{
		m_alarmstate=raisealarm;
		m_alarmhigh=high;
		m_alarmdate=DateTime::Now();
		m_alarmtext=m_description;
		if(!high && m_low_alarm_txt!="")m_alarmtext+=" " + m_low_alarm_txt;
		else if(high && m_high_alarm_txt!="")m_alarmtext+=" " + m_high_alarm_txt;		   
		
		FireAlarm(false);		
	}
	else if(raisealarm==0 && m_alarmstate!=0)
	{
		//alarm recover
		
		FireAlarm(true);		
		
		m_alarmstate=0;
		m_alarmtext="";		
	}
	
	
	
	return true;	
}



void Monitor::FireAlarm(bool recover)
{
	string state;
	switch(m_alarmstate)
	{
		case 1: state="Minor";break;
		case 2: state="Major";break;
		case 3: state="Critical";break;
		default: state="None";
	}
	
	
	stringstream alertss;	
	alertss<<"       Date: "<<DateTime::Now().GetDate()<<"\r\n       Host: "<<m_agentname<<"\r\nDescription: "<<m_description<<"\r\n      Value: "<<m_lastvalue<<"\r\n      Alarm: "<<m_alarmtext<<"\r\n      State: "<<state;	
	
	
	string emailbody = alertss.str();
	
	string emailsubject = "*ALARM* " + m_agentname + ": " + m_alarmtext + " " + state + (recover? " [Recover]" : "");
	
	
	Log(2,"fire alert: " + emailsubject);	
	
	if(m_conf->ValueExists("smtpserver"))
	{
		string smtpserver = m_conf->GetValueStr("smtpserver");
		string emailfrom = m_conf->GetValueStr("emailfrom");
		
		string email;
		for(int i=0;m_alert_emails.Get(email,i);i++)
		{
			EmailClient client(smtpserver);		
			
			if(emailfrom!="")client.SetFrom(emailfrom);
			
			client.SendTo(email,emailsubject,emailbody);
			Log(4,"sent email alert to " + email);
		}
	}
	
}


int Monitor::AlarmState()
{
	return m_alarmstate;
}

DateTime Monitor::AlarmDate()
{
	return m_alarmdate;
}

string Monitor::AlarmStateStr()
{
	return "";
}


bool Monitor::Dump2Disk()
{
	Log(4,"Dumping data to disk...");	
	
	
	fstream fileout(string("output/" + m_agentname + "/" + m_name + ".dat").c_str(),ios::out | ios::app);
	
	DataEntry * tmpentry=NULL;
	while(m_dataentries.PopFrom(tmpentry,0))
	{		
		Log(5,"Writing value to disk: " + BitStream::Int2Str(tmpentry->value));
		fileout<<tmpentry->date<<";"<<tmpentry->value<<"\r\n";
		
		delete tmpentry;
	}	
		
	fileout.close();	
	
	return true;
}