/**********************************************************
EnviroState C++ header file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-08-2012 10:34:14pm
Filepath: includes/Monitor.h
Lines of code: 61

Description: Monitor entry (server mode)

***********************************************************/


#ifndef Monitor_h
#define Monitor_h

class Core;



class Monitor
{
	friend class ServerCore;
	friend class Agent;
	friend class AgentNetSession;
	
	private:
		ServerCore * m_core; 			//core (declared in main program function)
		Agent * m_agent;		//agent this belongs to (declared in core)
		string m_agentname;		//agent name (block name in the config file)
		MlConfFile * m_conf;	//configuration (declared in core)
		string m_name;			//monitor name (block name in config file)
		
		xVector<DataEntry*> m_dataentries;	//contains actual data, this is occasionally flushed to disk
		
		//------- config settings ---------
		//---------------------------------
		
		string m_description;
		
		string m_low_alarm_txt;
		string m_high_alarm_txt;
		
		int m_pollinterval;
		
		string m_source;
		
		int m_sticky;
		int m_stickydir;
		
		float m_multiplier;
		
		bool m_ignoremin;	//ignore low values (only make sure it's below max)
		bool m_ignoremax;	//ignore high values (only make sure it's above min) 
		//NOTE: If both of the above are true then techncially alarm reporting as a whole is turned off
		
		int m_alm_min_min;
		int m_alm_min_max;
		int m_alm_maj_min;
		int m_alm_maj_max;
		int m_alm_crit_min;
		int m_alm_crit_max;	

		xVector<string> m_alert_emails;	//list of emails that get alarm alerts

		//------- end config settings ------
		//----------------------------------
		
		
		bool m_value_is_final;	//wether the current stored value is "final".  if false, no action taken on current value. (ex: at first startup the value will be wrong until a first polling of data occurs)
		bool m_virgin;			//no value has yet been set
		
		bool m_enable_alm_min;	//enable minor alarm reporting
		bool m_enable_alm_maj;	//enable major alarm reporting
		bool m_enable_alm_crit;	//enable critical alarm reporting
		
		int m_stickypolls; //number of polls where we kept the value sticky (aka skip reading data until this reaches value of m_sticky)
		
		DateTime m_lastchecked;
		int m_alarmstate;			//current alarm state
		bool m_alarmhigh;			//alarm sent was for high value
		DateTime m_alarmdate;		//date it went into alarm
		string m_alarmtext;			//text for the alarm
		
		int m_lastvalue;	//last value polled
		void FireAlarm(bool recover=false);		//this undergoes the task of sending email alert, and possibly other tasks in the future
	public:	
		Monitor(ServerCore * core,Agent * agent, string name);		
		~Monitor();
		void Log(int val,string txt);
		void Load(string confpath,MlConfFile * conf);	//loads from config file		
		
		bool IntervalReached();		//wether or not check interval has been reached
		
		int GetValue();		//returns stored value (assumes a poll was just made)
		void SetValue(int val,int date);	//sets value (this called when agent connects to set data)
		void SetValue(int val);				//sets value now
		
		bool Poll();		//polls data, returns false if not time to poll yet, true if it did poll
		
		int AlarmState(); 		//returns alarm state: 0: noalm, 1: min, 2: maj, 3: crit
		DateTime AlarmDate();	//returns date it went into alarm
		string AlarmStateStr();	//string of text with alarm state for the alarm list file
		bool Dump2Disk();		//dumps accumulated data to disk
	
};


#endif