/**********************************************************
EnviroState C++ source file
Enviromental Monitoring System
Last modified by Red Squirrel on Jul-29-2012 10:50:24pm
Filepath: includes/serialconnector/SerialConnector.cpp
Lines of code: 112

Description: 

***********************************************************/


SerialConnector::SerialConnector()
{
	m_Path="";
	m_conactive=false;
	DebugMode=false;
	m_fd=0;
}

SerialConnector::SerialConnector(string path)
{
	m_Path=path;
	m_conactive=false;
	DebugMode=false;
	m_fd=0;
}


void SerialConnector::ScreenOutput(string output)
{
	//if(DebugMode)LOGGER->Log(5, m_Path + ": " + output);
}




void SerialConnector::SetPath(string path)
{
	m_Path=path;
}


bool SerialConnector::Open()
{
	if(m_Path=="")
	{
		ScreenOutput("No path specified");
		return false;
	}


	
	//init stuff
	
	struct termios tio;


	memset(&tio,0,sizeof(tio));
	tio.c_iflag=0;
	tio.c_oflag=0;
	tio.c_cflag=CS8|CREAD|CLOCAL;           // 8n1, see termios.h for more information
	tio.c_lflag=0;
	tio.c_cc[VMIN]=1;
	tio.c_cc[VTIME]=5;


	//connect
	m_fd=open(m_Path.c_str(), O_RDWR | O_NONBLOCK);	
	
	
	
    cfsetospeed(&tio,B115200);            // 115200 baud
    cfsetispeed(&tio,B115200);            // 115200 baud	

	//this is required, it seems that it only needs to be called once per server reboot. It sets some kind of global thing that prevents the serial device (in my test: uk1104 from canakit) to go insane and spam junk
	tcsetattr(m_fd,TCSANOW,&tio);
	
	if (m_fd == -1 )
	{
		ScreenOutput("Unable to connect");
		m_conactive=false;
		return false;
	}

	m_conactive=true;
	
	return true;
}


void SerialConnector::Close()
{
	m_conactive=false;
	close(m_fd);
}


bool SerialConnector::ConActive()
{
	return m_conactive;
}


string SerialConnector::ReadLine(bool waitindef,bool nlonly)
{
	string ret;	

	if(!m_conactive)return "";
	
	//read a single line:
	
	char buff[1]; //array (the read takes an array, not a single char)
	int bytes=0;
	
	//TODO: Add some kind of safety guard to avoid getting stuck in an infinite "no data" loop 
	
		
	while(1)
	{
		bytes= read(m_fd,buff,1);
		
		if(bytes<0)
		{
			if(errno==11)
			{
				if(waitindef)
				{
					continue; //keep trying
				}
				break; //EAGAIN: No more data			
			}
			
			ScreenOutput("Error while reading.  Errno:" + BitStream::Int2Str(errno));
			return "";
		}
		else if(bytes==0)
		{
			//lost connection
			ScreenOutput("Lost connection");
			m_conactive=false;
			break; 
		}
		
		
		if(buff[0]=='\r')
		{
			if(!nlonly)
			{
				read(m_fd,buff,1);//read the following char which should be \n
				if(buff[0]=='\n')break;
			}
			else break;			
		}
		else
		{				
			ret.append(1,buff[0]);  			
		}
		
		
		if(ret.size()>1024) //hard coded limit to catch issue where device goes haywire and loops forever.  
		{
			break;
		}
		
	}
	
	return ret;
}


bool SerialConnector::WriteLine(string line,bool nlonly)
{
	if(!m_conactive)return false;
	
	string towrite = line + (nlonly? "\n" : "\r\n");
	
	return write(m_fd,towrite.c_str(),towrite.size());
}

























